@@ -24,7 +24,6 @@
 #endif
  
 #include <stdarg.h>
-#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -37,20 +36,37 @@
 #include "lang.h"
 #include "intrf.h"
 #include "fnctdsk.h"
+#include "chgtype.h"
 #include "testdisk.h"
 #include "fat.h"
 #include "ntfs.h"
 #include "adv.h"
+#include "analyse.h"
+#include "intrface.h"
+
 #define INTER_ADV_X	0
 #define INTER_ADV_Y	23
 #define INTER_ADV	15
 
+static int is_linux(const int part_type);
+
+static int is_linux(const int part_type)
+{
+  switch(part_type)
+  {
+    case P_LINUX:
+      return 1;
+  }
+  return 0;
+}
+
 void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, const unsigned int expert)
 {
   int quit;
   int offset=0;
   int current_element_num=0;
   int rewrite=1;
+  const char *options;
   t_list_part *element;
   t_list_part *list_part;
   t_list_part *current_element;
@@ -65,11 +81,13 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
   {
     static struct MenuItem menuAdv[]=
     {
-      {'q',"Quit","Return to main menu"},
+      {'t',"Type","Change type, this settings will not be saved"},
       {'b',"Boot","Boot sector recovery"},
+      {'s',"Superblock","Locate EXT2/EXT3 backup superblock"},
+      {'q',"Quit","Return to main menu"},
       {0,NULL,NULL}
     };
-    int opt_B=0;
+    int menu=0;
     int i;
     int command;
     if(rewrite!=0)
@@ -95,15 +113,26 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
 	aff_part(stdscr,AFF_PART_ORDER,disk_car,element->part);
       }
     }
+    options="tq";
+    menu=0;
     if(current_element!=NULL)
     {
+      /* TODO: is_fat, is_ntfs, is_linux only works on i386 */
       if(is_fat(current_element->part->part_type) ||
 	  is_ntfs(current_element->part->part_type))
-	opt_B=1;
+      {
+	options="tbq";
+	menu=1;
+      }
+      else if(is_linux(current_element->part->part_type))
+      {
+	options="tsq";
+	menu=1;
+      }
     }
     quit=0;
-    command = wmenuSelect(stdscr,INTER_ADV_Y, INTER_ADV_X, menuAdv, 8,
-	opt_B!=0?"bq":"q", MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, opt_B);
+    command = wmenuSelect(stdscr,INTER_ADV_Y, INTER_ADV_X, menuAdv, 8, options,
+	MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu);
     switch(command)
     {
       case MENU_UP:
@@ -155,6 +184,24 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
 	  }
 	}
 	break;
+      case 's':
+      case 'S':
+	if(current_element!=NULL)
+	{
+	  t_list_part *list_sb=search_superblock(disk_car,current_element->part,debug,dump_ind,1);
+	  interface_superblock(disk_car,list_sb);
+	  delete_list_part(list_sb);
+	  rewrite=1;
+	}
+	break;
+      case 't':
+      case 'T':
+	if(current_element!=NULL)
+	{
+	  change_part_type(disk_car,current_element->part);
+	  rewrite=1;
+	}
+	break;
     }
   } while(quit==0);
   delete_list_part(list_part);
@@ -162,7 +209,7 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
 
 int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert)
 {
-  t_sector buffer_bs;
+  unsigned char buffer_bs[0x200];
   const char *options="DR";
   int rescan=1;
   while(1)
@@ -178,10 +225,15 @@ int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
       aff_part(stdscr,AFF_PART_ORDER,disk_car,partition);
       ecrit_rapport("\nfat1x_boot_sector\n");
       aff_part_rapport(disk_car,partition);
+      if(expert==0)
+	options="DRC";
+      else
+	options="DRCI";
       aff_buffer(BUFFER_ADD,"Boot sector\n");
       if(disk_car->read(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset)!=0)
       {
-	ecrit_rapport(msg_CHKFAT_RERR); return 1;
+	aff_buffer(BUFFER_ADD,"fat1x_boot_sector: Can't read boot sector.\n");
+	memset(&buffer_bs,0,sizeof(buffer_bs));
       }
       if(test_FAT(disk_car,(const struct fat_boot_sector *)buffer_bs,partition,debug,0)==0)
       {
@@ -211,13 +263,18 @@ int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
 	  mvwaddstr(window,6,0, "Boot sector");
 	  dump(window,buffer_bs,DEFAULT_SECTOR_SIZE);
 	  delwin(window);
-#ifdef DJGPP
-	  wredrawln(stdscr,0,stdscr->_maxy);	/* redrawwin def is boggus in pdcur24 */
-#else
-	  redrawwin(stdscr);	/* stdscr has been corrupted by window */
+	  (void) clearok(stdscr, TRUE);
+#ifdef HAVE_TOUCHWIN
+	  touchwin(stdscr);
 #endif
 	}
 	break;
+      case 'C':
+	repair_FAT_table(disk_car,partition,debug);
+	break;
+      case 'I':
+	FAT_init_rootdir(disk_car,partition,debug);
+	break;
     }
   }
 }
@@ -226,7 +283,7 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
 {
   unsigned char buffer_bs[3*disk_car->sector_size];
   unsigned char buffer_backup_bs[3*disk_car->sector_size];
-  const char *options="";
+  const char *options="DRC";
   int rescan=1;
   while(1)
   {
@@ -236,10 +293,7 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
       int opt_over=0;
       int opt_B=0;
       int opt_O=0;
-      if(expert==0)
-	options="DR";
-      else
-	options="DRC";
+      options="DRC";
       aff_copy(stdscr);
       wmove(stdscr,4,0);
       wdoprintf(stdscr,"%s",disk_car->description(disk_car));
@@ -251,7 +305,8 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
       aff_buffer(BUFFER_ADD,"Boot sector\n");
       if(disk_car->read(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset)!=0)
       {
-	ecrit_rapport(msg_CHKFAT_RERR); return 1;
+	aff_buffer(BUFFER_ADD,"fat32_boot_sector: Can't read boot sector.\n");
+	memset(&buffer_bs,0,sizeof(buffer_bs));
       }
       if(test_FAT(disk_car,(struct fat_boot_sector *)buffer_bs,partition,debug,0)==0)
       {
@@ -262,7 +317,8 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
       aff_buffer(BUFFER_ADD,"\nBackup boot sector\n");
       if(disk_car->read(disk_car,sizeof(buffer_backup_bs), &buffer_backup_bs, partition->part_offset+6*disk_car->sector_size)!=0)
       {
-	ecrit_rapport(msg_CHKFAT_RERR); return 1;
+	aff_buffer(BUFFER_ADD,"fat32_boot_sector: Can't read backup boot sector.\n");
+	memset(&buffer_backup_bs,0,sizeof(buffer_backup_bs));
       }
       if(test_FAT(disk_car,(struct fat_boot_sector *)buffer_backup_bs,partition,debug,0)==0)
       {
@@ -340,16 +396,15 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
 	  mvwaddstr(window,6,0, "Boot sector                        Backup boot sector");
 	  dump2(window,buffer_bs,buffer_backup_bs,3*disk_car->sector_size);
 	  delwin(window);
-#ifdef DJGPP
-	  wredrawln(stdscr,0,stdscr->_maxy);	/* redrawwin def is boggus in pdcur24 */
-#else
-	  redrawwin(stdscr);	/* stdscr has been corrupted by window */
+	  (void) clearok(stdscr, TRUE);
+#ifdef HAVE_TOUCHWIN
+	  touchwin(stdscr);
 #endif
 	  dump_2fat_rapport((const struct fat_boot_sector*)&buffer_bs,(const struct fat_boot_sector*)&buffer_backup_bs,UP_FAT32,disk_car->sector_size);
 	}
 	break;
       case 'C':
-	clean_FAT(disk_car,partition,debug);
+	repair_FAT_table(disk_car,partition,debug);
 	break;
     }
   }
@@ -381,7 +436,8 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
       aff_buffer(BUFFER_ADD,"Boot sector\n");
       if(disk_car->read(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset)!=0)
       {
-	ecrit_rapport(msg_CHKFAT_RERR); return 1;
+	aff_buffer(BUFFER_ADD,"ntfs_boot_sector: Can't read boot sector.\n");
+	memset(&buffer_bs,0,sizeof(buffer_bs));
       }
       if(test_NTFS(disk_car,(struct ntfs_boot_sector*)buffer_bs,partition,debug,0)==0)
       {
@@ -396,7 +452,8 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
       aff_buffer(BUFFER_ADD,"\nBackup boot sector\n");
       if(disk_car->read(disk_car,sizeof(buffer_backup_bs), &buffer_backup_bs, partition->part_offset+partition->part_size-disk_car->sector_size)!=0)
       {
-	ecrit_rapport(msg_CHKFAT_RERR); return 1;
+	aff_buffer(BUFFER_ADD,"ntfs_boot_sector: Can't read backup boot sector.\n");
+	memset(&buffer_backup_bs,0,sizeof(buffer_backup_bs));
       }
       if(test_NTFS(disk_car,(struct ntfs_boot_sector*)buffer_backup_bs,partition,debug,0)==0)
       {
@@ -474,10 +531,9 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
 	  mvwaddstr(window,6,0, "Boot sector                        Backup boot sector");
 	  dump2(window,buffer_bs,buffer_backup_bs,DEFAULT_SECTOR_SIZE);
 	  delwin(window);
-#ifdef DJGPP
-	  wredrawln(stdscr,0,stdscr->_maxy);	/* redrawwin def is boggus in pdcur24 */
-#else
-	  redrawwin(stdscr);	/* stdscr has been corrupted by window */
+	  (void) clearok(stdscr, TRUE);
+#ifdef HAVE_TOUCHWIN
+	  touchwin(stdscr);
 #endif
 	}
 	break;
