@@ -26,5 +26,7 @@ int search_type_2(t_param_disk *disk_car,t_partition *partition,const int debug,
 int search_type_16(t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind);
 int search_type_64(t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind);
 int search_type_128(t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind);
-int search_NTFS_backup(t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind);
 int search_FAT_backup(t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind);
+int search_HFS_backup(t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind);
+int search_NTFS_backup(t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind);
+t_list_part *search_superblock(t_param_disk *disk_car, const t_partition *partition, const int debug, const int dump_ind, const int interface);
