@@ -24,7 +24,6 @@
 #include <config.h>
 #endif
  
-#include <stdio.h>
 #include "types.h"
 #include "common.h"
 #include "bsd.h"
@@ -60,16 +59,16 @@ int test_BSD(t_param_disk *disk_car, const struct disklabel*bsd_header,t_partiti
     for(cp=(const uint16_t*)bsd_header;
 	cp<(const uint16_t*)&bsd_header->d_partitions[le16(bsd_header->d_npartitions)];cp++)
       crc^=*cp;
-    if(debug)
+    if(crc==0)
     {
-      if(!crc)
+      if(debug)
+      {
 	ecrit_rapport("CRC Ok\n");
-      else
-	ecrit_rapport("Bad CRC! CRC must be xor'd by %04X\n",crc);
+      }
     }
-    if(crc)
-      return 1;
-    for(i=0;i<max_partitions;i++)
+    else
+      ecrit_rapport("Bad CRC! CRC must be xor'd by %04X\n",crc);
+    for(i=0;i<bsd_header->d_npartitions;i++)
     {
       if(bsd_header->d_partitions[i].p_fstype>0)
       {
@@ -102,6 +101,8 @@ int test_BSD(t_param_disk *disk_car, const struct disklabel*bsd_header,t_partiti
 	}
       }
     }
+    if(crc)
+      return 1;
     if(max_partitions==BSD_MAXPARTITIONS)
       partition->upart_type=UP_FREEBSD;
     else
@@ -135,6 +136,9 @@ int recover_BSD(t_param_disk *disk_car, const struct disklabel*bsd_header,t_part
     else
       partition->part_size=0;
     partition->part_type=P_FREEBSD;
+    set_part_name(partition,bsd_header->d_packname,16);
+    if(check_volume_name(partition->name,16))
+      partition->name[0]='\0';
     return 0;
   }
   if(test_BSD(disk_car,bsd_header,partition,debug,dump_ind,OPENBSD_MAXPARTITIONS)==0)
@@ -153,6 +157,9 @@ int recover_BSD(t_param_disk *disk_car, const struct disklabel*bsd_header,t_part
     else
       partition->part_size=0;
     partition->part_type=P_OPENBSD;
+    set_part_name(partition,bsd_header->d_packname,16);
+    if(check_volume_name(partition->name,16))
+      partition->name[0]='\0';
     return 0;
   }
   return 1;
