@@ -21,8 +21,8 @@
  */
 /* Come mainly from sys/disklabel.h and disktab.h */
 
-#ifndef BSDDL_H
-#define BSDDL_H
+#ifndef _BSD_H
+#define _BSD_H
 
 #define STANDALONE
 
