@@ -23,18 +23,9 @@
 #include <config.h>
 #endif
  
-#include <stdarg.h>
-#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
-#include <ctype.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>	/* geteuid */
-#endif
 #include "types.h"
 #include "common.h"
 #include "lang.h"
@@ -51,8 +42,8 @@ int change_part_type(const t_param_disk *disk_car,t_partition *partition)
   aff_buffer(BUFFER_ADD,"List of partition type\n");
   for(i=0;i<=0xFF;i++)
   {
-    const char *part_name;
-    part_name=partition_type(i, disk_car->arch->parttype_name_table);
+    const unsigned char *part_name;
+    part_name=partition_type(i, ((partition->arch!=NULL)?partition->arch->parttype_name_table:disk_car->arch->parttype_name_table));
     if(part_name!=NULL)
     {
       aff_buffer(BUFFER_ADD,"%02x %-20s ",i,part_name);
