@@ -25,7 +25,6 @@
 #include <config.h>
 #endif
  
-#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -87,6 +86,38 @@ int vsnprintf(char *str, size_t size, const char *format, va_list ap)
 }
 #endif
 
+#ifndef HAVE_STRNCASECMP
+/** Case-insensitive, size-constrained, lexical comparison. 
+ *
+ * Compares a specified maximum number of characters of two strings for 
+ * lexical equivalence in a case-insensitive manner.
+ *
+ * @param[in] s1 - The first string to be compared.
+ * @param[in] s2 - The second string to be compared.
+ * @param[in] len - The maximum number of characters to compare.
+ * 
+ * @return Zero if at least @p len characters of @p s1 are the same as
+ *    the corresponding characters in @p s2 within the ASCII printable 
+ *    range; a value less than zero if @p s1 is lexically less than
+ *    @p s2; or a value greater than zero if @p s1 is lexically greater
+ *    than @p s2.
+ *
+ * @internal
+ */
+int strncasecmp(const char * s1, const char * s2, size_t len)
+{
+  while (*s1 && (*s1 == *s2 || tolower(*s1) == tolower(*s2)))
+  {
+    len--;
+    if (len == 0) 
+      return 0;
+    s1++;
+    s2++;
+  }
+  return (int)*(const unsigned char *)s1 - (int)*(const unsigned char *)s2;
+}
+#endif
+
 unsigned int up2power(const unsigned int number)
 {
 /*  ecrit_rapport("up2power(%u)=>%u\n",number, (1<<up2power_aux(number-1))); */
@@ -138,7 +169,5 @@ void set_part_name(t_partition *partition,const char *src,const int max_size)
   for(i=0;(i<max_size) && (src[i]!=(char)0);i++)
     partition->name[i]=src[i];
   partition->name[i--]='\0';
-  for(;(i>=0) && (src[i]==' ');i--);
-  partition->name[i+1]='\0';
 }
 
