@@ -57,7 +57,10 @@
 #define P_FREEBSD       0xA5
 #define P_OPENBSD       0xA6
 #define P_NETBSD        0xA9
+#define P_HFS		0xAF
+#define P_HFSP		0xAF
 #define P_BEOS          0xEB
+#define P_SUN		0xBF
 #define P_RAID		0xFD
 #define NO_ORDER 	255
 /* Partition SUN */
@@ -68,11 +71,13 @@
 #define PSUN_STAND	6
 #define PSUN_VAR	7
 #define PSUN_HOME	8
+#define PSUN_ALT	9
+#define PSUN_CACHEFS	10
 #define PSUN_LINSWAP	P_LINSWAP
 #define PSUN_LINUX	P_LINUX
-#define P_SUN_LVM	P_LVM
+#define PSUN_LVM	P_LVM
 
-enum upart_type { UP_UNK, UP_FAT12, UP_FAT16, UP_FAT32, UP_EXTENDED, UP_NTFS, UP_HPFS, UP_OS2MB, UP_NETWARE, UP_EXT2, UP_EXT3, UP_LVM, UP_FREEBSD, UP_OPENBSD, UP_BEOS, UP_RAID, UP_LINSWAP, UP_LINSWAP2, UP_RFS, UP_RFS2, UP_RFS3, UP_JFS, UP_CRAMFS, UP_XFS, UP_XFS2, UP_XFS3, UP_XFS4, UP_UFS, UP_SYSV2, UP_SYSV4, UP_SYSV7};
+enum upart_type { UP_BEOS, UP_CRAMFS, UP_EXT2, UP_EXT3, UP_EXTENDED, UP_FAT12, UP_FAT16, UP_FAT32, UP_FREEBSD, UP_HPFS, UP_JFS, UP_HFS, UP_HFSP, UP_LINSWAP, UP_LINSWAP2, UP_NETWARE, UP_NTFS, UP_LVM, UP_LVM2, UP_OPENBSD, UP_OS2MB, UP_RAID, UP_RFS, UP_RFS2, UP_RFS3, UP_SUN, UP_SYSV2, UP_SYSV4, UP_SYSV7, UP_UFS, UP_UFS2, UP_UNK, UP_XFS, UP_XFS2, UP_XFS3, UP_XFS4};
 typedef enum upart_type upart_type_t;
 enum status_type { STATUS_DELETED, STATUS_PRIM, STATUS_PRIM_BOOT, STATUS_LOG, STATUS_EXT, STATUS_EXT_IN_EXT};
 typedef enum status_type status_type_t;
@@ -84,28 +89,8 @@ int ecrit_rapport(const char *_format, ...) __attribute__ ((format (printf, 1, 2
 void ecrit_rapport_string(const char *string,const int max_length);
 
 typedef struct cellule_disk_param t_param_disk;
-
-typedef unsigned char t_sector[0x200];
-typedef unsigned char t_entry[0x10];
 typedef struct cellule_diskext t_partition;
 typedef struct cellule_CHS t_CHS;
-#define pt_offset_const(b, n) ((const struct partition_dos *)((b) + 0x1be + \
-      (n) * sizeof(struct partition_dos)))
-#define pt_offset(b, n) ((struct partition_dos *)((b) + 0x1be + \
-      (n) * sizeof(struct partition_dos)))
-
-struct partition_dos {
-    unsigned char boot_ind;         /* 0x80 - active */
-    unsigned char head;             /* starting head */
-    unsigned char sector;           /* starting sector */
-    unsigned char cyl;              /* starting cylinder */
-    unsigned char sys_ind;          /* What partition type */
-    unsigned char end_head;         /* end head */
-    unsigned char end_sector;       /* end sector */
-    unsigned char end_cyl;          /* end cylinder */
-    unsigned char start4[4];        /* starting sector counting from 0 */
-    unsigned char size4[4];         /* nr of sectors in partition */
-};
 
 struct cellule_CHS
 {
@@ -137,7 +122,7 @@ struct systypes {
 
 struct arch_fnct
 {
-  const unsigned char *part_name;
+  const char *part_name;
   const struct systypes *parttype_name_table;
   t_list_part *(*read_part)(t_param_disk *disk_car, const int debug,const int saveheader);
   int (*write_part)(t_param_disk *disk_car, const t_list_part *list_part, const int ro, const int debug, const int align);
@@ -186,6 +171,7 @@ struct cellule_diskext
   errcode_type_t errcode;
   char name[80];
   char info[80];
+  const t_arch_fnct *arch;
 };
 
 typedef struct str_fat t_fat_test;
