@@ -24,7 +24,6 @@
 #include <config.h>
 #endif
  
-#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -75,7 +74,7 @@ int recover_cramfs(t_param_disk *disk_car, const struct cramfs_super *sb,t_parti
     }
   }
   partition->part_size = sb->size;
-  partition->part_type = (unsigned char)0x83;
+  partition->part_type = (unsigned char)P_LINUX;
   set_cramfs_info(disk_car,sb,partition,debug,dump_ind);
   return 0;
 }
