@@ -23,8 +23,6 @@
 #include <config.h>
 #endif
  
-#include <stdio.h>
-#include <ctype.h>	/* toupper */
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -35,9 +33,6 @@
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include "types.h"
 #include "common.h"
 #include "fat.h"
@@ -46,10 +41,12 @@
 #include "testdisk.h"
 #include "intrf.h"
 #include "dir.h"
-#define INTER_DIR 16
+
+static int dir_partition_aux(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode, const int first_time);
+static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const t_file_data*dir_list, const unsigned long int inode, const int first_time);
+
 static const char *monstr[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
 				"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
-static void mode_string (const unsigned int mode, char *str);
 static char ftypelet (unsigned int bits);
 
 static char ftypelet (unsigned int bits)
@@ -189,14 +186,14 @@ int dir_aff_log(const t_param_disk *disk_car, const t_partition *partition, cons
   return test_date;
 }
 
-long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const t_file_data*dir_list, const unsigned long int inode)
+static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const t_file_data*dir_list, const unsigned long int inode, const int first_time)
 {
   /* Return value
    * -1: quit
    *  1: back
    *  other: new inode
    * */
-  int quit;
+  int quit=0;
   int offset=0;
   int pos_num=0;
   const t_file_data *current_file;
@@ -204,7 +201,8 @@ long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_dat
   aff_copy(dir_data->window);
   wmove(dir_data->window,4,0);
   aff_part(dir_data->window,AFF_PART_ORDER,disk_car,partition);
-  mvwaddstr(dir_data->window,5,0,"Use arrow keys (<- and ->) to change directory, q to quit");
+  mvwaddstr(dir_data->window,5,0,(first_time!=0? "Use right arrow to change directory, q to quit":
+	"Use left arrow to go back, right arrow to change directory, q to quit"));
   wmove(dir_data->window,6,0);
   wdoprintf(dir_data->window,"Directory %s\n",dir_data->current_directory);
   do
@@ -235,18 +233,23 @@ long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_dat
       wdoprintf(dir_data->window, "%s %5u %5u   ", 
 	  str, (unsigned int)current_file->filestat.st_uid, (unsigned int)current_file->filestat.st_gid);
       wdoprintf(dir_data->window, "%7llu", (long long unsigned int)current_file->filestat.st_size);
-      /* FIXME: screen overlaps due to long filename */
-      wdoprintf(dir_data->window, " %s %s\n", datestr, current_file->name);
+      /* screen may overlap due to long filename */
+      wdoprintf(dir_data->window, " %s %s", datestr, current_file->name);
       if(current_file==pos)
 	wattroff(dir_data->window,A_STANDOUT);
     }
     /* Clear the last line, useful if overlapping */
     wmove(dir_data->window,8+i-offset,0);
     wclrtoeol(dir_data->window);	/* before addstr for BSD compatibility */
-
+    if(dir_list==NULL)
+    {
+      wmove(dir_data->window,8,0);
+      wdoprintf(dir_data->window,"No file found, damaged partition ?");
+    }
     wrefresh(dir_data->window);
+    /* Using gnome terminal under FC3, TERM=xterm, the screen is not always correct */
+    wredrawln(dir_data->window,0,getmaxy(dir_data->window));	/* redrawwin def is boggus in pdcur24 */
     car=wgetch(dir_data->window);
-    quit=0;
     switch(car)
     {
       case key_ESC:
@@ -256,7 +259,9 @@ long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_dat
 	break;
       case '-':
       case KEY_LEFT:
-	return 1;
+	if(first_time==0)
+	  return 1;
+	break;
     }
     if(dir_list!=NULL)
     {
@@ -344,49 +349,18 @@ void delete_list_file(t_file_data *file_list)
     current_file=next;
   }
 }
-/* #define TESTDIR 1 */
-#ifdef TESTDIR
+
 int dir_partition(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode)
 {
-  t_file_data *dir_list;
-  unsigned int current_directory_namelength=strlen(dir_data->current_directory);
-  t_file_data *current_file;
-  dir_list=dir_data->get_dir(disk_car,partition,dir_data,inode);
-  dir_aff_log(disk_car, partition, dir_data, dir_list);
-  for(current_file=dir_list;current_file!=NULL;current_file=current_file->next)
-  {
-    if(LINUX_S_ISDIR(current_file->filestat.st_mode)!=0)
-    {
-      unsigned long int new_inode=current_file->filestat.st_ino;
-      if(new_inode!=inode)
-      {
-	if((strcmp(current_file->name,"..")==0)||(strcmp(current_file->name,".")==0))
-	{
-	}
-	else
-	{
-	  if(strlen(dir_data->current_directory)+1+strlen(current_file->name)<sizeof(dir_data->current_directory)-1)
-	  {
-	    if(strcmp(dir_data->current_directory,"/"))
-	      strcat(dir_data->current_directory,"/");
-	    strcat(dir_data->current_directory,current_file->name);
-	    dir_partition(disk_car, partition, dir_data, (unsigned long int)new_inode);
-	    dir_data->current_directory[current_directory_namelength]='\0';
-	  }
-	}
-      }
-    }
-  }
-  delete_list_file(dir_list);
-  return 0;
+  if(dir_data==NULL)
+    return -1;
+  return dir_partition_aux(disk_car,partition,dir_data,inode,1);
 }
-#else
-int dir_partition(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode)
+
+static int dir_partition_aux(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode, const int first_time)
 {
   t_file_data *dir_list;
   long int new_inode;
-  if(dir_data==NULL)
-    return -1;
   if(dir_data->debug!=0)
     ecrit_rapport("\ndir_partition inode=%ld\n",inode);
   dir_list=dir_data->get_dir(disk_car,partition,dir_data,inode);
@@ -394,10 +368,10 @@ int dir_partition(t_param_disk *disk_car, const t_partition *partition, t_dir_da
   do
   {
     unsigned int current_directory_namelength=strlen(dir_data->current_directory);
-    new_inode=dir_aff(disk_car,partition,dir_data,dir_list,inode);
+    new_inode=dir_aff(disk_car,partition,dir_data,dir_list,inode,first_time);
     if(new_inode==0 || new_inode>1)
     {
-      if(dir_partition(disk_car, partition, dir_data, (unsigned long int)new_inode)<0)
+      if(dir_partition_aux(disk_car, partition, dir_data, (unsigned long int)new_inode,0)<0)
       { /* quit */
 	delete_list_file(dir_list);
 	return -1;
@@ -409,4 +383,3 @@ int dir_partition(t_param_disk *disk_car, const t_partition *partition, t_dir_da
   delete_list_file(dir_list);
   return new_inode;
 }
-#endif
