@@ -1,5 +1,6 @@
 typedef struct dir_data t_dir_data;
 typedef struct file_data t_file_data;
+#define INTER_DIR 16
 #define DIR_NAME_LEN 255
 struct dir_data
 {
@@ -19,9 +20,9 @@ struct file_data
 };
 
 int dir_aff_log(const t_param_disk *disk_car, const t_partition *partition, const t_dir_data *dir_data, const t_file_data*dir_list);
-long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const t_file_data*dir_list, const unsigned long int inode);
 void delete_list_file(t_file_data *file_list);
 int dir_partition(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode);
+void mode_string (const unsigned int mode, char *str);
 
 #define LINUX_S_IFMT  00170000
 #define LINUX_S_IFSOCK 0140000
