@@ -23,8 +23,6 @@
 #include <config.h>
 #endif
  
-#include <stdarg.h>
-#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -32,9 +30,6 @@
 #include <string.h>
 #endif
 #include <ctype.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include "types.h"
 #include "common.h"
 #include "lang.h"
