@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
  
-#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -35,6 +34,7 @@
 
 #define EXT2_BLOCK_SIZE(s)      (EXT2_MIN_BLOCK_SIZE << (s)->s_log_block_size)
 static int test_EXT2(t_param_disk *disk_car, const struct ext2_super_block *sb,t_partition *partition,const int debug, const int dump_ind);
+static int set_EXT2_info(t_param_disk *disk_car, const struct ext2_super_block *sb,t_partition *partition,const int debug, const int dump_ind);
 
 int check_EXT2(t_param_disk *disk_car,t_partition *partition,const int debug)
 {
@@ -47,7 +47,7 @@ int check_EXT2(t_param_disk *disk_car,t_partition *partition,const int debug)
   return 0;
 }
 
-int set_EXT2_info(t_param_disk *disk_car, const struct ext2_super_block *sb,t_partition *partition,const int debug, const int dump_ind)
+static int set_EXT2_info(t_param_disk *disk_car, const struct ext2_super_block *sb,t_partition *partition,const int debug, const int dump_ind)
 {
   partition->info[0]='\0';
   set_part_name(partition,sb->s_volume_name,16);
@@ -102,7 +102,7 @@ int recover_EXT2(t_param_disk *disk_car, const struct ext2_super_block *sb,t_par
     }
     partition->boot_sector=boot_sector;
     partition->part_offset-=(uint64_t)boot_sector*((EXT2_MIN_BLOCK_SIZE/disk_car->sector_size)<<le32(sb->s_log_block_size))*disk_car->sector_size;
-    ecrit_rapport("recover_EXT2: e2fsck -b %lu device may be needed\n",partition->boot_sector);
+    ecrit_rapport("recover_EXT2: \"e2fsck -b %lu -B %lu device\" may be needed\n",partition->boot_sector,partition->blocksize);
   }
   else
   {
@@ -121,6 +121,7 @@ int recover_EXT2(t_param_disk *disk_car, const struct ext2_super_block *sb,t_par
 
 static int test_EXT2(t_param_disk *disk_car, const struct ext2_super_block *sb,t_partition *partition,const int debug, const int dump_ind)
 {
+    /* There is a little offset ... */
   if(le16(sb->s_magic)!=EXT2_SUPER_MAGIC)
     return 1;
   if(dump_ind!=0)
