@@ -24,17 +24,13 @@
 #endif
  
 #ifdef HAVE_LIBEXT2FS
-#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#include <time.h>
+//#include <time.h>
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -72,7 +68,7 @@ static errcode_t my_write_blk(io_channel channel, unsigned long block, int count
 static errcode_t my_flush(io_channel channel);
 
 
-static io_channel alloc_io_channel(t_param_disk *disk_car,t_partition *partition);
+static io_channel alloc_io_channel(t_param_disk *disk_car,t_my_data *my_data);
 
 static struct struct_io_manager my_struct_manager = {
         magic: EXT2_ET_MAGIC_IO_MANAGER,
@@ -83,9 +79,13 @@ static struct struct_io_manager my_struct_manager = {
         read_blk: my_read_blk,
         write_blk: my_write_blk,
         flush: my_flush,
-	write_byte: NULL
+	write_byte: NULL,
+#ifdef HAVE_STRUCT_STRUCT_IO_MANAGER_SET_OPTION
+	set_option: NULL
+#endif
 };
 
+io_channel *shared_ioch=NULL;
 io_manager my_io_manager = &my_struct_manager;
 /*
  * Macro taken from unix_io.c
@@ -98,10 +98,9 @@ io_manager my_io_manager = &my_struct_manager;
 /*
  * Allocate libext2fs structures associated with I/O manager
  */
-static io_channel alloc_io_channel(t_param_disk *disk_car,t_partition *partition)
+static io_channel alloc_io_channel(t_param_disk *disk_car,t_my_data *my_data)
 {
   io_channel     ioch;
-  t_my_data	*my_data;
 #ifdef DEBUG
   ecrit_rapport("alloc_io_channel start\n");
 #endif
@@ -111,15 +110,12 @@ static io_channel alloc_io_channel(t_param_disk *disk_car,t_partition *partition
   memset(ioch, 0, sizeof(struct struct_io_channel));
   ioch->magic = EXT2_ET_MAGIC_IO_CHANNEL;
   ioch->manager = my_io_manager;
-  ioch->name = (char *)MALLOC(strlen(partition->name)+1);
+  ioch->name = (char *)MALLOC(strlen(my_data->partition->name)+1);
   if (ioch->name==NULL) {
 	  FREE(ioch);
 	  return NULL;
   }
-  strncpy(ioch->name, partition->name,strlen(partition->name)+1);
-  my_data=MALLOC(sizeof(*my_data));
-  my_data->partition=partition;
-  my_data->disk_car=disk_car;
+  memcpy(ioch->name, my_data->partition->name,strlen(my_data->partition->name)+1);
   ioch->private_data = my_data;
   ioch->block_size = 1024; /* The smallest ext2fs block size */
   ioch->read_error = 0;
@@ -132,9 +128,9 @@ static io_channel alloc_io_channel(t_param_disk *disk_car,t_partition *partition
 
 static errcode_t my_open(const char *dev, int flags, io_channel *channel)
 {
-  *channel = (io_channel)dev;
+  *channel = *shared_ioch;
 #ifdef DEBUG
-  ecrit_rapport("my_open done\n");
+  ecrit_rapport("my_open %s done\n",dev);
 #endif
   return 0;
 }
@@ -214,10 +210,11 @@ static int list_dir_proc(struct ext2_dir_entry *dirent,
   ino = dirent->inode;
 /*  ecrit_rapport("list_dir_proc inode %u\n",(unsigned)ino); */
   if (ino) {
-    if (ext2fs_read_inode(ls->current_fs,ino, &inode))
+    errcode_t retval;
+    if ((retval=ext2fs_read_inode(ls->current_fs,ino, &inode))!=0)
     {
-      FREE(new_file);
-      return 0;
+      ecrit_rapport("ext2fs_read_inode(ino=%u) failed with error %ld.\n",(unsigned)ino, (long)retval);
+      memset(&inode, 0, sizeof(struct ext2_inode));
     }
   } else {
     memset(&inode, 0, sizeof(struct ext2_inode));
@@ -272,12 +269,16 @@ static int list_dir_proc2(ext2_ino_t dir,
   new_file->name[thislen] = '\0';
   ino = dirent->inode;
   if (ino) {
+    errcode_t retval;
     ecrit_rapport("ext2fs_read_inode(ino=%u)\n",ino);
-    if (ext2fs_read_inode(ls->current_fs,ino, &inode))
+    if ((retval=ext2fs_read_inode(ls->current_fs,ino, &inode))!=0)
     {
-      ecrit_rapport("ext2fs_read_inode(ino=%u) failed\n",ino);
+      ecrit_rapport("ext2fs_read_inode(ino=%u) failed with error %ld.\n",(unsigned)ino, (long)retval);
+      memset(&inode, 0, sizeof(struct ext2_inode));
+      /*
       FREE(new_file);
       return 0;
+      */
     }
   } else {
     memset(&inode, 0, sizeof(struct ext2_inode));
@@ -310,11 +311,15 @@ static int list_dir_proc2(ext2_ino_t dir,
 t_file_data *ext2_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster);
 t_file_data *ext2_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster)
 {
+  errcode_t       retval;
   struct list_dir_struct *ls=(struct list_dir_struct*)dir_data->private_dir_data;
   ls->dir_list=NULL;
   ls->current_file=NULL;
-  ext2fs_dir_iterate(ls->current_fs, cluster, ls->flags, 0, list_dir_proc, ls);
 //  ext2fs_dir_iterate2(ls->current_fs, cluster, ls->flags, 0, list_dir_proc2, ls);
+  if((retval=ext2fs_dir_iterate(ls->current_fs, cluster, ls->flags, 0, list_dir_proc, ls))!=0)
+  {
+    ecrit_rapport("ext2fs_dir_iterate failed with error %ld.\n",(long)retval);
+  }
   return ls->dir_list;
 }
 
@@ -323,17 +328,24 @@ int dir_partition_ext2(WINDOW *window,t_param_disk *disk_car,t_partition *partit
   errcode_t       retval;
   static struct list_dir_struct ls;
   io_channel ioch;
+  t_my_data *my_data;
   ls.dir_list=NULL;
   ls.current_file=NULL;
   aff_buffer(BUFFER_RESET,"Q");
+  aff_copy(window);
   wmove(window,4,0);
   aff_part(window,AFF_PART_ORDER,disk_car,partition);
-  ioch=alloc_io_channel(disk_car,partition);
-  retval = ext2fs_open ((const char*)ioch, 0, partition->boot_sector, partition->blocksize, my_io_manager, &ls.current_fs);
-  if(retval)
+  my_data=MALLOC(sizeof(*my_data));
+  my_data->partition=partition;
+  my_data->disk_car=disk_car;
+
+  ioch=alloc_io_channel(disk_car,my_data);
+  shared_ioch=&ioch;
+  retval = ext2fs_open ("/dev/testdisk", 0, partition->boot_sector, partition->blocksize, my_io_manager, &ls.current_fs);
+  if(retval!=0)
   {
     aff_part_rapport(disk_car,partition);
-    aff_buffer(BUFFER_ADD,"Can't open ext2 filesystem (retval=%u)\n",retval);
+    aff_buffer(BUFFER_ADD,"Can't open ext2 filesystem (retval=%ld)\n",(long)retval);
     aff_buffer(BUFFER_DISPLAY,"Q",window);
     return -1;
   }
@@ -360,7 +372,8 @@ int dir_partition_ext2(WINDOW *window,t_param_disk *disk_car,t_partition *partit
 int dir_partition_ext2(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug)
 {
   aff_buffer(BUFFER_RESET,"Q");
-  wmove(stdscr,4,0);
+  aff_copy(window);
+  wmove(window,4,0);
   aff_part(window,AFF_PART_ORDER,disk_car,partition);
   aff_part_rapport(disk_car,partition);
   aff_buffer(BUFFER_ADD,"Recompile with e2fsprogs library");
