@@ -2,7 +2,7 @@
 
     File: fat.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include <config.h>
 #endif
  
-#include <stdio.h>
 #include <ctype.h>	/* toupper */
 #ifdef HAVE_STRING_H
 #include <string.h>
@@ -42,8 +41,20 @@
 #include "intrf.h"
 
 static int set_FAT_info(t_param_disk *disk_car, const struct fat_boot_sector *fat_header, t_partition *partition,const int debug);
+static void fat_set_part_name(t_partition *partition,const unsigned char *src,const int max_size);
+static int dump_fat_rapport(const struct fat_boot_sector*fh1, const upart_type_t upart_type, const unsigned int sector_size);
 
-int dump_fat_rapport(const struct fat_boot_sector*fh1, const upart_type_t upart_type, const unsigned int sector_size)
+static void fat_set_part_name(t_partition *partition,const unsigned char *src,const int max_size)
+{
+  int i;
+  for(i=0;(i<max_size) && (src[i]!=(char)0);i++)
+    partition->name[i]=src[i];
+  partition->name[i--]='\0';
+  for(;(i>=0) && (src[i]==' ');i--);
+  partition->name[i+1]='\0';
+}
+
+static int dump_fat_rapport(const struct fat_boot_sector*fh1, const upart_type_t upart_type, const unsigned int sector_size)
 {
   ecrit_rapport("sector_size  %u\n", fat_sector_size(fh1));
   ecrit_rapport("cluster_size %u\n", fh1->cluster_size);
@@ -157,10 +168,10 @@ int dump_2fat_info(const struct fat_boot_sector*fh1, const struct fat_boot_secto
     else
       wdoprintf(stdscr,"%lu ",fat32_get_free_count((const unsigned char*)fh1,sector_size));
     if(fat32_get_free_count((const unsigned char*)fh2,sector_size)==0xFFFFFFFF)
-      wdoprintf(stdscr,"uninitialised");
+      wdoprintf(stdscr,"uninitialised\n");
     else
-      wdoprintf(stdscr,"%lu",fat32_get_free_count((const unsigned char*)fh2,sector_size));
-    wdoprintf(stdscr,"\nnext_free    ");
+      wdoprintf(stdscr,"%lu\n",fat32_get_free_count((const unsigned char*)fh2,sector_size));
+    wdoprintf(stdscr,"next_free    ");
     if(fat32_get_next_free((const unsigned char*)fh1,sector_size)==0xFFFFFFFF)
       wdoprintf(stdscr,"uninitialised ");
     else
@@ -295,7 +306,7 @@ static int set_FAT_info(t_param_disk *disk_car, const struct fat_boot_sector *fa
       snprintf(partition->info,sizeof(partition->info),"FAT12");
       if(buffer[38]==0x29)	/* BS_BootSig */
       {
-	set_part_name(partition,((const char*)fat_header)+FAT1X_PART_NAME,11);
+	fat_set_part_name(partition,((const unsigned char*)fat_header)+FAT1X_PART_NAME,11);
 	if(check_volume_name(partition->name,11))
 	  partition->name[0]='\0';
       }
@@ -304,7 +315,7 @@ static int set_FAT_info(t_param_disk *disk_car, const struct fat_boot_sector *fa
       snprintf(partition->info,sizeof(partition->info),"FAT16");
       if(buffer[38]==0x29)	/* BS_BootSig */
       {
-	set_part_name(partition,((const char*)fat_header)+FAT1X_PART_NAME,11);
+	fat_set_part_name(partition,((const unsigned char*)fat_header)+FAT1X_PART_NAME,11);
 	if(check_volume_name(partition->name,11))
 	  partition->name[0]='\0';
       }
@@ -480,6 +491,7 @@ unsigned int get_prev_cluster(t_param_disk *disk_car,const t_partition *partitio
 
 int test_FAT(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_partition *partition,const int debug, const int dump_ind)
 {
+  upart_type_t upart_type=UP_UNK;
   unsigned long int start_fat1,start_fat2,start_rootdir,start_data,no_of_cluster,fat_length,fat_length_calc,part_size,end_data;
   const char *buffer=(const char*)fat_header;
   if(debug>1)
@@ -523,27 +535,35 @@ int test_FAT(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_
       ecrit_rapport(msg_CHKFAT_SECT_CLUSTER);
       return 1;
   }
-  if(fat_header->fats!=2)
+  switch(fat_header->fats)
   {
-    aff_buffer(BUFFER_ADD,msg_CHKFAT_NBRFAT);
-    ecrit_rapport(msg_CHKFAT_NBRFAT);
-    return 1;
+    case 1:
+      aff_buffer(BUFFER_ADD,"check_FAT: Unusual, only one FAT\n");
+      ecrit_rapport("check_FAT: Unusual, only one FAT\n");
+      break;
+    case 2:
+      break;
+    default:
+      aff_buffer(BUFFER_ADD,"check_FAT: Bad number %u of FAT\n", fat_header->fats);
+      ecrit_rapport("check_FAT: Bad number %u of FAT\n", fat_header->fats);
+      return 1;
   }
   if(fat_header->media!=0xF0 && fat_header->media<0xF8)
   {	/* Legal values are 0xF0, 0xF8-0xFF */
-    aff_buffer(BUFFER_ADD,"check_FAT: Bad media descriptor (0x%2x!=0xF8)\n",fat_header->media);
-    ecrit_rapport("check_FAT: Bad media descriptor (0x%2x!=0xF8)\n",fat_header->media);
+    aff_buffer(BUFFER_ADD,"check_FAT: Bad media descriptor (0x%2x!=0xf8)\n",fat_header->media);
+    ecrit_rapport("check_FAT: Bad media descriptor (0x%2x!=0xf8)\n",fat_header->media);
     return 1;
   }
   if(fat_header->media!=0xF8)
   { /* the only value I have ever seen is 0xF8 */
-    ecrit_rapport("check_FAT: Unusual media descriptor (0x%2x!=0xF8)\n",fat_header->media);
+    aff_buffer(BUFFER_ADD,"check_FAT: Unusual media descriptor (0x%2x!=0xf8)\n",fat_header->media);
+    ecrit_rapport("check_FAT: Unusual media descriptor (0x%2x!=0xf8)\n",fat_header->media);
   }
   fat_length=le16(fat_header->fat_length)>0?le16(fat_header->fat_length):le32(fat_header->fat32_length);
   part_size=(sectors(fat_header)>0?sectors(fat_header):fat_header->total_sect);
   start_fat1=le16(fat_header->reserved);
-  start_fat2=start_fat1+fat_length;
-  start_data=start_fat2+fat_length+(get_dir_entries(fat_header)*32+disk_car->sector_size-1)/disk_car->sector_size;
+  start_fat2=start_fat1+(fat_header->fats>1?fat_length:0);
+  start_data=start_fat1+fat_header->fats*fat_length+(get_dir_entries(fat_header)*32+disk_car->sector_size-1)/disk_car->sector_size;
   no_of_cluster=(part_size-start_data)/fat_header->cluster_size;
   end_data=start_data+no_of_cluster*fat_header->cluster_size-1;
   if(debug>1)
@@ -586,6 +606,7 @@ int test_FAT(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_
       aff_buffer(BUFFER_ADD,"Should be marked as FAT12\n");
       ecrit_rapport("Should be marked as FAT12\n");
     }
+    upart_type=UP_FAT12;
   }
   else if(no_of_cluster<65525)
   {
@@ -618,6 +639,7 @@ int test_FAT(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_
       aff_buffer(BUFFER_ADD,"Should be marked as FAT16\n");
       ecrit_rapport("Should be marked as FAT16\n");
     }
+    upart_type=UP_FAT16;
   }
   else
   {
@@ -656,6 +678,7 @@ int test_FAT(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_
       aff_buffer(BUFFER_ADD,"Should be marked as FAT32\n");
       ecrit_rapport("Should be marked as FAT32\n");
     }
+    upart_type=UP_FAT32;
   }
   if(partition->part_size>0)
   {
@@ -685,8 +708,12 @@ int test_FAT(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_
     ecrit_rapport("fat_length %lu calculated %lu\n",fat_length,fat_length_calc);
   }
   if(fat_length<fat_length_calc)
-  { aff_buffer(BUFFER_ADD,msg_CHKFAT_SECTPFAT); return 1; }
-  comp_FAT(disk_car,partition,fat_length,le16(fat_header->reserved));
+  {
+    aff_buffer(BUFFER_ADD,msg_CHKFAT_SECTPFAT);
+    return 1;
+  }
+  if(fat_header->fats>1)
+    comp_FAT(disk_car,partition,fat_length,le16(fat_header->reserved));
   if(le16(fat_header->secs_track)!=disk_car->CHS.sector)
   {
     aff_buffer(BUFFER_ADD,"check_FAT: Incorrect number of sectors per track %u (FAT) != %u (HD)\n",le16(fat_header->secs_track),disk_car->CHS.sector);
@@ -756,89 +783,67 @@ unsigned long int fat32_get_next_free(const unsigned char *boot_fat32, const uns
   return (boot_fat32[sector_size+0x1EC+3]<<24)+(boot_fat32[sector_size+0x1EC+2]<<16)+(boot_fat32[sector_size+0x1EC+1]<<8)+boot_fat32[sector_size+0x1EC];
 }
 
-int recover_FAT12(t_param_disk *disk_car,const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind)
+int recover_FAT(t_param_disk *disk_car, const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind, const int backup)
 {
   const char*buffer=(const char*)fat_header;
-  if(le16(fat_header->marker)==0xAA55)
+  if(le16(fat_header->marker)==0xAA55
+      && (fat_header->ignored[0]==0xeb || fat_header->ignored[0]==0xe9)
+      && (fat_header->fats==1 || fat_header->fats==2))
   {
-    if(memcmp(buffer+FAT_NAME1,"FAT12",5)==0) /* 2 Mo max */
+    if(test_FAT(disk_car, fat_header, partition, debug, dump_ind))
+      return 1;
+    /* test_FAT has set partition->upart_type */
+    switch(partition->upart_type)
     {
-      if(debug||dump_ind)
-      {
-	ecrit_rapport("\nFAT12 at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
-      }
-      if(test_FAT(disk_car, fat_header, partition, debug, dump_ind))
-	return 1;
-      partition->part_size=(uint64_t)sectors(fat_header)*fat_sector_size(fat_header);
-      partition->part_type=P_12FAT;
-      set_FAT_info(disk_car,fat_header,partition,debug);
-      return 0;
-    }
-  }     /* fin marqueur de fin :)) */
-  return 1;
-}
-
-int recover_FAT16(t_param_disk *disk_car,const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind)
-{
-  const char*buffer=(const char*)fat_header;
-  if(le16(fat_header->marker)==0xAA55)
-  {
-    if(memcmp(buffer+FAT_NAME1,"FAT16",5)==0)
-    {
-      if(debug||dump_ind)
-      {
-	ecrit_rapport("\nFAT16 at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
-      }
-      if(test_FAT(disk_car, fat_header, partition, debug, dump_ind))
-	return 1;
-      if(sectors(fat_header)!=0)
-      {
+      case UP_FAT12:
+	if(debug||dump_ind)
+	{
+	  ecrit_rapport("\nFAT12 at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
+	}
 	partition->part_size=(uint64_t)sectors(fat_header)*fat_sector_size(fat_header);
-	partition->part_type=P_16FAT;
-      } else {
-	partition->part_size=(uint64_t)le32(fat_header->total_sect)*fat_sector_size(fat_header);
-	if(offset2cylinder(disk_car,partition->part_offset+partition->part_size-1)<=1024)
-	  partition->part_type=P_16FATBD;
+	partition->part_type=P_12FAT;
+	break;
+      case UP_FAT16:
+	if(debug||dump_ind)
+	{
+	  ecrit_rapport("\nFAT16 at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
+	}
+	if(sectors(fat_header)!=0)
+	{
+	  partition->part_size=(uint64_t)sectors(fat_header)*fat_sector_size(fat_header);
+	  partition->part_type=P_16FAT;
+	} else {
+	  partition->part_size=(uint64_t)le32(fat_header->total_sect)*fat_sector_size(fat_header);
+	  if(offset2cylinder(disk_car,partition->part_offset+partition->part_size-1)<=1024)
+	    partition->part_type=P_16FATBD;
+	  else
+	    partition->part_type=P_16FATBD_LBA;
+	}
+	break;
+      case UP_FAT32:
+	if(debug||dump_ind)
+	{
+	  ecrit_rapport("\nFAT32 at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
+	}
+	if(sectors(fat_header)!=0)
+	  partition->part_size=(uint64_t)sectors(fat_header)*fat_sector_size(fat_header);
 	else
-	  partition->part_type=P_16FATBD_LBA;
-      }
-      set_FAT_info(disk_car,fat_header,partition,debug);
-      return 0;
-    }
-  }     /* fin marqueur de fin :)) */
-  return 1;
-}
-
-int recover_FAT32(t_param_disk *disk_car, const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind,const int backup)
-{
-  const char*buffer=(const char*)fat_header;
-  if(le16(fat_header->marker)==0xAA55)
-  {
-    if(memcmp(buffer+FAT_NAME2,"FAT32",5)==0)
-    {
-      if(debug||dump_ind)
-      {
-	ecrit_rapport("\nFAT32 at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
-      }
-      if(test_FAT(disk_car, fat_header, partition, debug, dump_ind))
+	  partition->part_size=(uint64_t)le32(fat_header->total_sect)*fat_sector_size(fat_header);
+	if(offset2cylinder(disk_car,partition->part_offset+partition->part_size-1)<=1024)
+	  partition->part_type=P_32FAT;
+	else
+	  partition->part_type=P_32FAT_LBA;
+	if(backup)
+	{
+	  partition->boot_sector=6;
+	  partition->part_offset-=6*512;	/* backup sector ... */
+	}
+	break;
+      default:
 	return 1;
-      if(sectors(fat_header)!=0)
-	partition->part_size=(uint64_t)sectors(fat_header)*fat_sector_size(fat_header);
-      else
-	partition->part_size=(uint64_t)le32(fat_header->total_sect)*fat_sector_size(fat_header);
-      if(offset2cylinder(disk_car,partition->part_offset+partition->part_size-1)<=1024)
-	partition->part_type=P_32FAT;
-      else
-	partition->part_type=P_32FAT_LBA;
-/*     partition->upart_type=UP_FAT32;	already done in test_FAT */
-      if(backup)
-      {
-	partition->boot_sector=6;
-	partition->part_offset-=6*512;	/* backup sector ... */
-      }
-      set_FAT_info(disk_car,fat_header,partition,debug);
-      return 0;
     }
+    set_FAT_info(disk_car,fat_header,partition,debug);
+    return 0;
   }     /* fin marqueur de fin =:-) */
   return 1;
 }
@@ -851,7 +856,7 @@ int fat32_set_part_name(t_param_disk *disk_car, t_partition *partition, const st
     unsigned char *buffer=(unsigned char*)MALLOC(fat_header->cluster_size*disk_car->sector_size);
     if(disk_car->read(disk_car,fat_header->cluster_size*disk_car->sector_size, buffer, partition->part_offset+(uint64_t)(le16(fat_header->reserved)+fat_header->fats*le32(fat_header->fat32_length)+(le32(fat_header->root_cluster)-2)*fat_header->cluster_size)*disk_car->sector_size))
     {
-      ecrit_rapport(msg_ROOT_CLUSTER_RERR);
+      ecrit_rapport("fat32_set_part_name, can not read FAT32 root cluster.\n");
     }
     else
     {
@@ -862,7 +867,7 @@ int fat32_set_part_name(t_param_disk *disk_car, t_partition *partition, const st
 	if(((buffer[i*0x20+0xB] & ATTR_EXT) !=ATTR_EXT) && ((buffer[i*0x20+0xB] & ATTR_VOLUME) !=0) && (buffer[i*0x20]!=0xE5))
 	{
 	  /*      dump(stdscr,&buffer[i*0x20],0x20); */
-	  set_part_name(partition,&buffer[i*0x20],11);
+	  fat_set_part_name(partition,&buffer[i*0x20],11);
 	  if(check_volume_name(partition->name,11))
 	    partition->name[0]='\0';
 	}
@@ -877,7 +882,7 @@ int fat32_set_part_name(t_param_disk *disk_car, t_partition *partition, const st
   if(partition->name[0]=='\0')
   {
     ecrit_rapport("set_FAT_info: name from BS used\n");
-    set_part_name(partition,((const char*)fat_header)+FAT32_PART_NAME,11);
+    fat_set_part_name(partition,((const unsigned char*)fat_header)+FAT32_PART_NAME,11);
     if(check_volume_name(partition->name,11))
       partition->name[0]='\0';
   }
