@@ -20,8 +20,8 @@
 
  */
 
-#ifndef FAT_H
-#define FAT_H
+#ifndef _FAT_H
+#define _FAT_H
 #define FAT1X_PART_NAME 0x2B
 #define FAT32_PART_NAME 0x47
 #define FAT_NAME1       0x36
@@ -102,20 +102,13 @@ struct msdos_dir_slot {
 int check_FAT(t_param_disk *disk_car,t_partition *partition, const int debug);
 int test_FAT(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_partition *partition,const int debug, const int dump_ind);
 int comp_FAT(t_param_disk *disk_car,const t_partition *partition, const unsigned long int taille, const unsigned long int sect_res);
-unsigned int get_frag(const unsigned char *buffer, const int fat_type);
-int dump_fat_rapport(const struct fat_boot_sector*fh1, const upart_type_t upart_type, const unsigned int sector_size);
 int dump_2fat_rapport(const struct fat_boot_sector*fh1, const struct fat_boot_sector*fh2, const upart_type_t upart_type, const unsigned int sector_size);
-int sort_fat(int pos_fat_test, int debug);
-int search_fat_aux(t_param_disk *disk_car,const t_partition *partition,const unsigned long int max_offset,const int p_fat12,const int p_fat16,const int p_fat32,int *f_fat12,int*f_fat16,int*f_fat32,const int debug,const int dump_ind,const int interface);
 
-unsigned long int get_subdirectory(t_param_disk *disk_car,const uint64_t hd_offset, const unsigned long int i);
 unsigned int get_next_cluster(t_param_disk *disk_car,const t_partition *partition, const upart_type_t upart_type,const int offset, const unsigned int cluster);
 unsigned int get_prev_cluster(t_param_disk *disk_car,const t_partition *partition, const upart_type_t upart_type,const int offset, const unsigned int cluster, const unsigned int no_of_cluster);
 int set_next_cluster(t_param_disk *disk_car,const t_partition *partition, const upart_type_t upart_type,const int offset, const unsigned int cluster, const unsigned int next_cluster);
 
-int recover_FAT12(t_param_disk *disk_car,const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind);
-int recover_FAT16(t_param_disk *disk_car,const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind);
-int recover_FAT32(t_param_disk *disk_car,const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind, const int backup);
+int recover_FAT(t_param_disk *disk_car,const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind, const int backup);
 int recover_HPFS(t_param_disk *disk_car, const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind);
 int recover_OS2MB(t_param_disk *disk_car, const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind);
 int is_fat(const int part_type);
@@ -134,7 +127,8 @@ unsigned int fat32_get_prev_cluster(t_param_disk *disk_car,const t_partition *pa
 int fat32_free_info(t_param_disk *disk_car,const t_partition *partition, const unsigned int fat_offset, const unsigned int no_of_cluster, unsigned int *next_free, unsigned int*free_count);
 unsigned long int fat32_get_free_count(const unsigned char *boot_fat32, const unsigned int sector_size);
 unsigned long int fat32_get_next_free(const unsigned char *boot_fat32, const unsigned int sector_size);
-int clean_FAT(t_param_disk *disk_car, t_partition *partition, const int debug);
+int repair_FAT_table(t_param_disk *disk_car, t_partition *partition, const int debug);
+int FAT_init_rootdir(t_param_disk *disk_car, t_partition *partition, const int debug);
 
 #define DELETED_FLAG 0xe5 /* marks file as deleted when in name[0] */
 #define IS_FREE(n) (!*(n) || *(const unsigned char *) (n) == DELETED_FLAG)
@@ -151,5 +145,7 @@ int clean_FAT(t_param_disk *disk_car, t_partition *partition, const int debug);
 #define ATTR_EXT     (ATTR_RO | ATTR_HIDDEN | ATTR_SYS | ATTR_VOLUME)
 #define ATTR_EXT_MASK     (ATTR_RO | ATTR_HIDDEN | ATTR_SYS | ATTR_VOLUME | ATTR_DIR | ATTR_ARCH)
 	/* bits that are used by the Windows 95/Windows NT extended FAT */
-#endif
+#define FAT12_EOC	0x0FF8
+#define FAT16_EOC	0xFFF8
 #define FAT32_EOC	0x0FFFFFF8
+#endif
