@@ -2,7 +2,7 @@
 
     File: dir_fat.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include <config.h>
 #endif
  
-#include <stdio.h>
 #include <ctype.h>	/* toupper */
 #ifdef HAVE_STRING_H
 #include <string.h>
@@ -36,9 +35,6 @@
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include "types.h"
 #include "common.h"
 #include "fat.h"
@@ -59,7 +55,7 @@ static inline void fat16_towchar(wchar_t *dst, const uint8_t *src, size_t len)
 	}
 }
 
-t_file_data *dir_fat_aux(const char*buffer, const unsigned int size, const unsigned int cluster_size)
+t_file_data *dir_fat_aux(const unsigned char*buffer, const unsigned int size, const unsigned int cluster_size)
 {
   const struct msdos_dir_entry *de=(const struct msdos_dir_entry*)buffer;
   wchar_t unicode[1000];
@@ -162,12 +158,14 @@ RecEnd:
       new_file->filestat.st_gid=0;
       new_file->filestat.st_rdev=0;
       new_file->filestat.st_size=le32(de->size);
+#ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
       new_file->filestat.st_blksize=cluster_size;
 #ifdef HAVE_STRUCT_STAT_ST_BLOCKS
       if(new_file->filestat.st_blksize!=0)
       {
 	new_file->filestat.st_blocks=(new_file->filestat.st_size+new_file->filestat.st_blksize-1)/new_file->filestat.st_blksize;
       }
+#endif
 #endif
       new_file->filestat.st_atime=new_file->filestat.st_ctime=new_file->filestat.st_mtime=date_dos2unix(le16(de->time),le16(de->date));
       new_file->prev=current_file;
@@ -214,10 +212,10 @@ int date_dos2unix(const unsigned short f_time, const unsigned short f_date)
 
 t_file_data *fat12_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_cluster)
 {
-  struct fat_boot_sector*fat_header=(struct fat_boot_sector*)dir_data->private_dir_data;
+  const struct fat_boot_sector*fat_header=(const struct fat_boot_sector*)dir_data->private_dir_data;
   if(fat_header->cluster_size<1)
   {
-    ecrit_rapport(msg_ROOT_CLUSTER_RERR);
+    ecrit_rapport("FAT12: Can't list files, bad cluster size\n");
     return NULL;
   }
   if(fat_sector_size(fat_header)==0)
@@ -236,17 +234,17 @@ t_file_data *fat12_dir(t_param_disk *disk_car, const t_partition *partition, t_d
     int stop=0;
     memset(buffer_dir,0,fat_sector_size(fat_header)*cluster_size*5);
     for(cluster=first_cluster, nbr_cluster=0;
-	((cluster&0x0ff8)!=(unsigned)0x0ff8) && (cluster>=2) && (nbr_cluster<5) && (stop==0);
+	((cluster&0x0ff8)!=(unsigned)FAT12_EOC) && (cluster>=2) && (nbr_cluster<5) && (stop==0);
 	cluster=get_next_cluster(disk_car,partition, UP_FAT12,le16(fat_header->reserved), cluster), nbr_cluster++)
     {
       uint64_t start=partition->part_offset+(uint64_t)(le16(fat_header->reserved)+fat_header->fats*le16(fat_header->fat_length)+(get_dir_entries(fat_header)*32+fat_sector_size(fat_header)-1)/fat_sector_size(fat_header)+(cluster-2)*cluster_size)*fat_sector_size(fat_header);
       if(dir_data->debug>0)
       {
-	ecrit_rapport("FAT12 cluster=%u(0x%x), pos=%lu\n",cluster,cluster,(long unsigned)(start/fat_sector_size(fat_header)));
+	ecrit_rapport("FAT12: cluster=%u(0x%x), pos=%lu\n",cluster,cluster,(long unsigned)(start/fat_sector_size(fat_header)));
       }
       if(disk_car->read(disk_car, cluster_size*fat_sector_size(fat_header), buffer_dir+(uint64_t)fat_sector_size(fat_header)*cluster_size*nbr_cluster, start))
       {
-	ecrit_rapport(msg_ROOT_CLUSTER_RERR);
+	ecrit_rapport("FAT12: Can't read directory cluster\n");
 	stop=1;
       }
     }
@@ -258,10 +256,10 @@ t_file_data *fat12_dir(t_param_disk *disk_car, const t_partition *partition, t_d
 
 t_file_data *fat16_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_cluster)
 {
-  struct fat_boot_sector*fat_header=(struct fat_boot_sector*)dir_data->private_dir_data;
+  const struct fat_boot_sector*fat_header=(const struct fat_boot_sector*)dir_data->private_dir_data;
   if(fat_header->cluster_size<1)
   {
-    ecrit_rapport(msg_ROOT_CLUSTER_RERR);
+    ecrit_rapport("FAT16: Can't list files, bad cluster size.\n");
     return NULL;
   }
   if(fat_sector_size(fat_header)==0)
@@ -281,7 +279,7 @@ t_file_data *fat16_dir(t_param_disk *disk_car, const t_partition *partition, t_d
     memset(buffer_dir,0,disk_car->sector_size*cluster_size*5);
     /* Need to correct the test */
     for(cluster=first_cluster, nbr_cluster=0;
-	((cluster&0xfff8)!=(unsigned)0xfff8) && (cluster>=2) && (nbr_cluster<5)&&(stop==0);
+	((cluster&0xfff8)!=(unsigned)FAT16_EOC) && (cluster>=2) && (nbr_cluster<5)&&(stop==0);
 	cluster=get_next_cluster(disk_car,partition, UP_FAT16,le16(fat_header->reserved), cluster), nbr_cluster++)
     {
       uint64_t start=partition->part_offset+(uint64_t)(le16(fat_header->reserved)+fat_header->fats*le16(fat_header->fat_length)+(get_dir_entries(fat_header)*32+disk_car->sector_size-1)/disk_car->sector_size+(cluster-2)*cluster_size)*disk_car->sector_size;
@@ -291,7 +289,7 @@ t_file_data *fat16_dir(t_param_disk *disk_car, const t_partition *partition, t_d
       }
       if(disk_car->read(disk_car, cluster_size*disk_car->sector_size, buffer_dir+(uint64_t)disk_car->sector_size*cluster_size*nbr_cluster, start))
       {
-	ecrit_rapport(msg_ROOT_CLUSTER_RERR);
+	ecrit_rapport("FAT16: Can't read directory cluster\n");
 	stop=1;
       }
     }
@@ -303,7 +301,7 @@ t_file_data *fat16_dir(t_param_disk *disk_car, const t_partition *partition, t_d
 
 t_file_data *fat32_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_cluster)
 {
-  struct fat_boot_sector*fat_header=(struct fat_boot_sector*)dir_data->private_dir_data;
+  const struct fat_boot_sector*fat_header=(const struct fat_boot_sector*)dir_data->private_dir_data;
   if(fat_header->cluster_size==0)
   {
     ecrit_rapport("FAT32: Can't list files, bad cluster size.\n");
@@ -329,7 +327,7 @@ t_file_data *fat32_dir(t_param_disk *disk_car, const t_partition *partition, t_d
     memset(buffer_dir,0,fat_sector_size(fat_header)*cluster_size*5);
     /* Need to correct the test */
     for(cluster=(first_cluster==0?le32(fat_header->root_cluster):first_cluster), nbr_cluster=0;
-	((cluster&0xffffff8)!=(unsigned)0xffffff8) && (cluster>=2) && (nbr_cluster<5) && (stop==0);
+	((cluster&0xffffff8)!=(unsigned)FAT32_EOC) && (cluster>=2) && (nbr_cluster<5) && (stop==0);
 	cluster=get_next_cluster(disk_car,partition, UP_FAT32,le16(fat_header->reserved), cluster), nbr_cluster++)
     {
       uint64_t start=partition->part_offset+(uint64_t)(le16(fat_header->reserved)+fat_header->fats*le32(fat_header->fat32_length)+(cluster-2)*cluster_size)*fat_sector_size(fat_header);
@@ -339,7 +337,7 @@ t_file_data *fat32_dir(t_param_disk *disk_car, const t_partition *partition, t_d
       }
       if(disk_car->read(disk_car, cluster_size*fat_sector_size(fat_header), buffer_dir+(uint64_t)fat_sector_size(fat_header)*cluster_size*nbr_cluster, start))
       {
-	ecrit_rapport(msg_ROOT_CLUSTER_RERR);
+	ecrit_rapport("FAT32: Can't read directory cluster\n");
 	stop=1;
       }
 //      ecrit_rapport("read cluster %u\n",cluster);
@@ -352,7 +350,7 @@ t_file_data *fat32_dir(t_param_disk *disk_car, const t_partition *partition, t_d
 }
 
 
-t_file_data *fat1x_rootdir(t_param_disk *disk_car, const t_partition *partition, const int debug, struct fat_boot_sector*fat_header)
+t_file_data *fat1x_rootdir(t_param_disk *disk_car, const t_partition *partition, const int debug, const struct fat_boot_sector*fat_header)
 {
   unsigned int root_size=(get_dir_entries(fat_header)*32+disk_car->sector_size-1)/disk_car->sector_size*disk_car->sector_size;
   if(debug>1)
@@ -367,8 +365,7 @@ t_file_data *fat1x_rootdir(t_param_disk *disk_car, const t_partition *partition,
     start=partition->part_offset+(uint64_t)((le16(fat_header->reserved)+fat_header->fats*le16(fat_header->fat_length))*disk_car->sector_size);
     if(disk_car->read(disk_car, root_size, buffer_dir, start))
     {
-      ecrit_rapport("FAT 1x");
-      ecrit_rapport(msg_ROOT_CLUSTER_RERR);
+      ecrit_rapport("FAT 1x: Can't read root directory\n");
       FREE(buffer_dir);
       return NULL;
     }
@@ -378,13 +375,13 @@ t_file_data *fat1x_rootdir(t_param_disk *disk_car, const t_partition *partition,
   }
 }
 
-int dir_partition_fat_aux(WINDOW *window,t_param_disk *disk_car, t_partition *partition, const int debug, struct fat_boot_sector*fat_header)
+int dir_partition_fat_aux(WINDOW *window,t_param_disk *disk_car, const t_partition *partition, const int debug, struct fat_boot_sector*fat_header, const upart_type_t upart_type)
 {
   t_dir_data dir_data;
   dir_data.window=window;
   dir_data.debug=debug;
   dir_data.private_dir_data=fat_header;
-  switch(partition->upart_type)
+  switch(upart_type)
   {
     case UP_FAT12:
       dir_data.get_dir=fat12_dir;
@@ -396,6 +393,7 @@ int dir_partition_fat_aux(WINDOW *window,t_param_disk *disk_car, t_partition *pa
       dir_data.get_dir=fat32_dir;
       break;
     default:
+      ecrit_rapport("Not a valid type (upart_type=%u)\n",partition->upart_type);
       return -1;
   }
   strncpy(dir_data.current_directory,"/",sizeof(dir_data.current_directory));
@@ -404,14 +402,18 @@ int dir_partition_fat_aux(WINDOW *window,t_param_disk *disk_car, t_partition *pa
 
 int dir_partition_fat(WINDOW *window,t_param_disk *disk_car, t_partition *partition, const int debug)
 {
-  t_sector buffer;
+  unsigned char buffer[0x200];
   if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+(uint64_t)partition->boot_sector*disk_car->sector_size))
   {
+    aff_buffer(BUFFER_RESET,"Q");
+    aff_copy(window);
+    wmove(window,4,0);
+    aff_part(window,AFF_PART_ORDER,disk_car,partition);
     aff_part_rapport(disk_car,partition);
     aff_buffer(BUFFER_ADD,"dir_partition_fat: Can't read FAT boot sector\n");
     aff_buffer(BUFFER_DISPLAY,"Q",window);
     return -1;
   }
-  dir_partition_fat_aux(window,disk_car, partition, debug, (struct fat_boot_sector*)buffer);
+  dir_partition_fat_aux(window,disk_car, partition, debug, (struct fat_boot_sector*)buffer,partition->upart_type);
   return 0;
 }
