@@ -1,12 +1,12 @@
-t_file_data *dir_fat_aux(const char*buffer, const unsigned int size, const unsigned int cluster_size);
+t_file_data *dir_fat_aux(const unsigned char*buffer, const unsigned int size, const unsigned int cluster_size);
 
 #define MSDOS_MKMODE(a,m) ((m & (a & ATTR_RO ? LINUX_S_IRUGO|LINUX_S_IXUGO : LINUX_S_IRWXUGO)) | (a & ATTR_DIR ? LINUX_S_IFDIR : LINUX_S_IFREG))
 
 int date_dos2unix(const unsigned short f_time,const unsigned short f_date);
-t_file_data *fat1x_rootdir(t_param_disk *disk_car, const t_partition *partition, const int debug, struct fat_boot_sector*fat_header);
+t_file_data *fat1x_rootdir(t_param_disk *disk_car, const t_partition *partition, const int debug, const struct fat_boot_sector*fat_header);
 t_file_data *fat12_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_cluster);
 t_file_data *fat16_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_cluster);
 t_file_data *fat32_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_cluster);
-int dir_partition_fat_aux(WINDOW *window,t_param_disk *disk_car, t_partition *partition, const int debug, struct fat_boot_sector*fat_header);
+int dir_partition_fat_aux(WINDOW *window,t_param_disk *disk_car, const t_partition *partition, const int debug, struct fat_boot_sector*fat_header, const upart_type_t upart_type);
 
 
