@@ -28,7 +28,6 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -39,7 +38,7 @@
 #include "testdisk.h"
 #include "intrf.h"
 #include "analyse.h"
-static void get_geometry_from_list_part_aux(const t_param_disk *disk_car, const t_list_part *list_part, const int debug);
+static unsigned int get_geometry_from_list_part_aux(const t_param_disk *disk_car, const t_list_part *list_part, const int debug);
 
 unsigned long int CHS2LBA(const t_param_disk *disk_car,const t_CHS*CHS)
 { return ((unsigned long int)CHS->cylinder*(disk_car->CHS.head+1)+CHS->head)*disk_car->CHS.sector+CHS->sector-1;
@@ -95,6 +94,7 @@ void dup_t_partition(t_partition *dest, const t_partition *src)
   dest->errcode=src->errcode;
   strncpy(dest->info,src->info,sizeof(dest->info));
   strncpy(dest->name,src->name,sizeof(dest->name));
+  dest->arch=src->arch;
 }
 
 int read_MBR(t_param_disk *disk_car,void *buffer)
@@ -139,18 +139,36 @@ t_list_disk *insert_new_disk(t_list_disk *list_disk, t_param_disk *disk_car)
 
 t_list_part *insert_new_partition(t_list_part *list_part, t_partition *part)
 {
-  return insert_new_partition_aux(list_part, element_new(part));
+  t_list_part *new_list_part;
+#ifdef DEBUG
+  check_list_part(list_part);
+#endif
+  new_list_part=insert_new_partition_aux(list_part, element_new(part),0);
+#ifdef DEBUG
+  check_list_part(new_list_part);
+#endif
+  return new_list_part;
 }
 
-t_list_part *insert_new_partition_aux(t_list_part *list_part, t_list_part *new_element)
-{ /* new partition musn't be used after insert !*/
-  t_list_part *prev=NULL;
+t_list_part *insert_new_partition_aux(t_list_part *list_part, t_list_part *new_element, const unsigned int dont_free_part)
+{
+/*dont_free_part
+  0 if new_element->part is already known, don't insert it and free it
+  1 if new_element->part is already known, don't insert it but free it
+  2 even if new_element->part is already known, insert it
+*/
+ /* new partition musn't be used after insert !*/  t_list_part *prev=NULL;
   t_list_part *next;
   for(next=list_part;;next=next->next)
   { /* prev new next */
-    if((next==NULL)||(new_element->part->part_offset<next->part->part_offset)||((new_element->part->part_offset==next->part->part_offset)&&(new_element->part->part_size<=next->part->part_size)))
+    if((next==NULL)||
+      (new_element->part->part_offset<next->part->part_offset) ||
+      (new_element->part->part_offset==next->part->part_offset &&
+       ((new_element->part->part_size<next->part->part_size) ||
+	(new_element->part->part_size==next->part->part_size && (dont_free_part!=2 || new_element->part->boot_sector<next->part->boot_sector)))))
     {
-      if((next!=NULL)&&(next->part->part_offset==new_element->part->part_offset) &&
+      if(dont_free_part!=2 &&
+	(next!=NULL)&&(next->part->part_offset==new_element->part->part_offset) &&
 	(next->part->part_size==new_element->part->part_size) &&
 	(next->part->part_type==new_element->part->part_type) &&
 	(next->part->upart_type==new_element->part->upart_type || new_element->part->upart_type==UP_UNK))
@@ -159,7 +177,8 @@ t_list_part *insert_new_partition_aux(t_list_part *list_part, t_list_part *new_e
 	{
 	  next->part->status=new_element->part->status;
 	}
-	FREE(new_element->part);
+	if(dont_free_part==0)
+	  FREE(new_element->part);
 	FREE(new_element);
 	return list_part;
       }
@@ -190,9 +209,10 @@ int check_list_part(t_list_part *list_part)
     ecrit_rapport("\ncheck_list_part error: list_part->prev!=NULL\n");
     exit(EXIT_FAILURE);
   }
+  ecrit_rapport("check_list_part\n");
   for(parts=list_part;parts!=NULL;parts=parts->next)
   {
-/*ecrit_rapport("%p %p %p\n",parts->prev, parts, parts->next); */
+    ecrit_rapport("%p %p %p\n",parts->prev, parts, parts->next);
 	if(prev!=parts->prev)
 	{
 	  ecrit_rapport("\ncheck_list_part error: prev!=parts->prev\n");
@@ -213,17 +233,26 @@ t_list_part *sort_list_part(t_list_part *list_part)
   t_list_part *new_list_part=NULL;
   t_list_part *element;
   t_list_part *next;
+#ifdef DEBUG
+  check_list_part(list_part);
+#endif
   for(element=list_part;element!=NULL;element=next)
   {
     next=element->next;
-    new_list_part=insert_new_partition_aux(new_list_part,element);
+    new_list_part=insert_new_partition_aux(new_list_part,element,0);
   }
+#ifdef DEBUG
+  check_list_part(new_list_part);
+#endif
   return new_list_part;
 }
 
 void delete_list_part(t_list_part *list_part)
 {
   t_list_part *element;
+#ifdef DEBUG
+  check_list_part(list_part);
+#endif
   /* Libere la memoire */
   element=list_part;
   while(element!=NULL)
@@ -248,6 +277,7 @@ void  partition_reset(t_partition *partition)
   partition->errcode=BAD_NOERR;
   partition->name[0]='\0';
   partition->info[0]='\0';
+  partition->arch=NULL;
 }
 
 t_partition *partition_new()
@@ -266,7 +296,7 @@ t_list_part *element_new(t_partition *part)
   return new_element;
 }
 
-static void get_geometry_from_list_part_aux(const t_param_disk *disk_car, const t_list_part *list_part, const int debug)
+static unsigned int get_geometry_from_list_part_aux(const t_param_disk *disk_car, const t_list_part *list_part, const int debug)
 {
   const t_list_part *element;
   unsigned int nbr=0;
@@ -300,21 +330,29 @@ static void get_geometry_from_list_part_aux(const t_param_disk *disk_car, const
       }
     }
   }
+  return nbr;
 }
 
-void get_geometry_from_list_part(const t_param_disk *disk_car, const t_list_part *list_part, const int debug)
+unsigned int get_geometry_from_list_part(const t_param_disk *disk_car, const t_list_part *list_part, const int debug)
 {
+  const unsigned int head_list[]={8,16,32,64,128,240,255,0};
+  unsigned int nbr_max;
+  unsigned int nbr;
+  unsigned int h_index=0;
+  unsigned int head_max=disk_car->CHS.head;
   t_param_disk *new_disk_car=MALLOC(sizeof(*new_disk_car));
   memcpy(new_disk_car,disk_car,sizeof(*new_disk_car));
-  for(new_disk_car->CHS.head=8-1;new_disk_car->CHS.head<=128-1;new_disk_car->CHS.head=2*(new_disk_car->CHS.head+1)-1)
+  nbr_max=get_geometry_from_list_part_aux(new_disk_car, list_part, debug);
+  for(h_index=0;head_list[h_index]!=0;h_index++)
   {
-    get_geometry_from_list_part_aux(new_disk_car, list_part, debug);
+    new_disk_car->CHS.head=head_list[h_index]-1;
+    nbr=get_geometry_from_list_part_aux(new_disk_car, list_part, debug);
+    if(nbr>=nbr_max)
+    {
+      nbr_max=nbr;
+      head_max=new_disk_car->CHS.head;
+    }
   }
-  new_disk_car->CHS.head=128-1;
-  get_geometry_from_list_part_aux(new_disk_car, list_part, debug);
-  new_disk_car->CHS.head=240-1;
-  get_geometry_from_list_part_aux(new_disk_car, list_part, debug);
-  new_disk_car->CHS.head=255-1;
-  get_geometry_from_list_part_aux(new_disk_car, list_part, debug);
   FREE(new_disk_car);
+  return head_max;
 }
