@@ -31,7 +31,7 @@ void dup_t_CHS(t_CHS * CHS_dest, const t_CHS * CHS_source);
 t_list_part *element_new(t_partition *part);
 t_list_disk *insert_new_disk(t_list_disk *list_disk, t_param_disk *disk_car);
 t_list_part *insert_new_partition(t_list_part *list_part, t_partition *part);
-t_list_part *insert_new_partition_aux(t_list_part *list_part, t_list_part *new_element);
+t_list_part *insert_new_partition_aux(t_list_part *list_part, t_list_part *new_element, const unsigned int dont_free_part);
 t_list_part *remove_partition(t_list_part *list_part, t_list_part *element2removed);
 t_list_part *sort_list_part(t_list_part *list_part);
 void delete_list_part(t_list_part *list_part);
@@ -43,5 +43,5 @@ unsigned int offset2head(const t_param_disk *disk_car, const uint64_t offset);
 unsigned int offset2cylinder(const t_param_disk *disk_car, const uint64_t offset);
 void offset2CHS(const t_param_disk *disk_car,const uint64_t offset, t_CHS*CHS);
 int check_list_part(t_list_part *list_part);
-void get_geometry_from_list_part(const t_param_disk *disk_car, const t_list_part *list_part, const int debug);
+unsigned int get_geometry_from_list_part(const t_param_disk *disk_car, const t_list_part *list_part, const int debug);
 
