@@ -2,7 +2,7 @@
 
     File: geometry.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,8 +23,6 @@
 #include <config.h>
 #endif
  
-#include <stdarg.h>
-#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -32,9 +30,6 @@
 #include <string.h>
 #endif
 #include <ctype.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include "types.h"
 #include "common.h"
 #include "intrf.h"
@@ -74,7 +69,7 @@ void change_geometry(t_param_disk *disk_car)
 		mvwaddstr(stdscr,COMMAND_LINE_Y, COMMAND_LINE_X, "Enter the number of cylinders: ");
 		if (get_string(response, LINE_LENGTH, def) > 0) {
 		  tmp_val = atoi(response);
-		  if (tmp_val > 0 && tmp_val <= MAX_CYLINDERS) {
+		  if (tmp_val > 0) {
 			disk_car->CHS.cylinder = tmp_val-1;
 		  } else
 			wdoprintf(stdscr,"Illegal cylinders value");
@@ -138,6 +133,8 @@ void change_geometry(t_param_disk *disk_car)
 		done = TRUE;
 		break;
 	}
+	if(modified!=0)
+	  disk_car->disk_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
   }
   disk_car->disk_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
   ecrit_rapport("New geometry\n%s sector_size=%u\n", disk_car->description(disk_car), disk_car->sector_size);
