@@ -2,7 +2,7 @@
 
     File: godmode.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -45,12 +45,16 @@
 #include "adv.h"
 #define RO 1
 #define RW 0
+#define MAX_SEARCH_LOCATION 1024
+extern const t_arch_fnct arch_none;
 static void align_structure(const t_param_disk *disk_car, t_list_part *list_part,const int align);
 static void ask_mbr_order(t_param_disk *disk_car,t_list_part *list_part);
 static t_list_part *reduce_structure(t_list_part *list_part);
 static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part, const int max_ext, const int align,const int debug);
 static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, const int debug,const int dump_ind, const unsigned int expert);
 static int interface_part_bad(t_param_disk *disk_car,t_list_part *list_part_bad);
+static void warning_geometry(t_param_disk *disk_car, const unsigned int recommanded_heads_per_cylinder);
+static t_list_part *search_part_raid(t_param_disk *disk_car, t_list_part *list_part, t_list_part *list_part_bad, const int debug, const int dump_ind);
 
 static void align_structure(const t_param_disk *disk_car, t_list_part *list_part, const int align)
 {
@@ -211,9 +215,94 @@ static int interface_part_bad(t_param_disk *disk_car,t_list_part *list_part)
   return 0;
 }
 
+static void warning_geometry(t_param_disk *disk_car, const unsigned int recommanded_heads_per_cylinder)
+{
+  int quit=0;
+  aff_copy(stdscr);
+  wmove(stdscr,4,0);
+  wdoprintf(stdscr,"%s",disk_car->description(disk_car));
+  wmove(stdscr,6,0);
+  ecrit_rapport("Warning: the current number of heads per cylinder is %u but the correct value may be %u.\n",disk_car->CHS.head+1,recommanded_heads_per_cylinder);
+  wdoprintf(stdscr,"Warning: the current number of heads per cylinder is %u but the correct value may be %u.\n",disk_car->CHS.head+1,recommanded_heads_per_cylinder);
+  wmove(stdscr,7,0);
+  wdoprintf(stdscr,"Use the Geometry menu to change this value.\n");
+  wmove(stdscr,22,0);
+  wattrset(stdscr,A_STANDOUT);
+  wdoprintf(stdscr,"[  Quit  ]");
+  wattroff(stdscr,A_STANDOUT);
+  wrefresh(stdscr);
+  do
+  {
+    int car;
+    car=wgetch(stdscr);
+    switch(car)
+    {
+      case 'q':
+      case '\r':
+      case '\n':
+      case KEY_ENTER:
+#ifdef PADENTER
+      case PADENTER:
+#endif
+      case 'M':
+	quit=1;
+	break;
+      default:
+/*	ecrit_rapport("ask_structure car=%x\n",car); */
+	break;
+    }
+  } while(quit==0);
+}
+
+static t_list_part *search_part_raid(t_param_disk *disk_car, t_list_part *list_part, t_list_part *list_part_bad, const int debug, const int dump_ind)
+{
+  t_list_part *element;
+  /* Little trick to detect Linux software Raid quickly */
+  for(element=list_part_bad;element!=NULL;element=element->next)
+  {
+    if(element->part->upart_type!=UP_RAID)
+    {
+      unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+      struct mdp_superblock_s *sb=(struct mdp_superblock_s*)&buffer;
+      unsigned int disk_factor;
+      unsigned int help_factor;
+      for(disk_factor=1;disk_factor<=6;disk_factor++)
+      { /* disk_factor=1, detect Raid 1 */
+        /* disk_factor>1, detect Raid 5 */
+	for(help_factor=0;help_factor<=MD_MAX_CHUNK_SIZE/MD_RESERVED_BYTES+3;help_factor++)
+	{
+	  uint64_t offset=MD_NEW_SIZE_SECTORS((element->part->part_size/disk_factor+help_factor*MD_RESERVED_BYTES-1)/MD_RESERVED_BYTES*MD_RESERVED_BYTES/512)*512;
+	  if(debug>1)
+	  {
+	    ecrit_rapport("search_part_raid offset=%llu\n",(long long unsigned)offset);
+	  }
+	  if(disk_car->read(disk_car,sizeof(buffer), &buffer, element->part->part_offset+offset)==0)
+	  {
+	    t_partition *new_partition=partition_new();
+	    dup_t_partition(new_partition,element->part);
+	    if(test_MD(disk_car,sb,new_partition,debug,dump_ind)==0)
+	    {
+	      set_MD_info(disk_car,sb,new_partition,debug,dump_ind);
+	      new_partition->part_type=P_RAID;
+	      new_partition->part_size=offset+MD_RESERVED_BYTES;
+	      aff_part_rapport(disk_car,new_partition);
+	      list_part=insert_new_partition(list_part,new_partition);
+	    }
+	    else
+	      FREE(new_partition);
+	  }
+	}
+      }
+    }
+  }
+  return list_part;
+}
+
 t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int debug, const int dump_ind, const int fast_mode,const int interface)
 {
   int ind_stop=0;
+  uint64_t try_offset[MAX_SEARCH_LOCATION];
+  unsigned int try_offset_nbr=0;
   t_CHS start;
   t_list_part *list_part=NULL;
   t_list_part *list_part_bad=NULL;
@@ -228,8 +317,8 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
   aff_buffer(BUFFER_RESET,"Q");
   ecrit_rapport("\nsearch_part()\n");
   ecrit_rapport("%s\n",disk_car->description(disk_car));
-  /* ncurses: getch will be a non-blocking call */
-  for(start.cylinder=0,start.head=1;(start.cylinder<=disk_car->CHS.cylinder)&&(ind_stop==0);start.cylinder++,start.head=0)
+  start.head=(disk_car->arch==&arch_none?0:1);
+  for(start.cylinder=0;(start.cylinder<=disk_car->CHS.cylinder)&&(ind_stop==0);start.cylinder++,start.head=0)
   {
     if(interface!=0)
     {
@@ -261,29 +350,40 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
       for(start.sector=1,sector_inc=1,part_offset=0;(start.sector<=disk_car->CHS.sector);start.sector+=sector_inc)
       {
 	int res=0;
+	unsigned int i;
+	int search_now=0;
 	partition_reset(partition);	/* must not reset lba */
 	partition->part_offset=CHS2offset(disk_car,&start);
-	if((start.sector==1) && ((start.head<=2)||(fast_mode==2)))
+	for(i=0;i<try_offset_nbr;i++)
+	{
+	  if(try_offset[i]==partition->part_offset)
+	  {
+	    search_now=1;
+	  }
+	}
+	if((start.sector==1) && ((start.head<=2)||(fast_mode==2)||(search_now>0)))
 	{
 	  part_offset++;
 	  switch(part_offset)
 	  {
 	    case 1:
-	      res=search_type_128(disk_car,partition,debug,dump_ind);
 	      sector_inc=0;
-	      break;
-	    case 2:
 	      res=search_type_64(disk_car,partition,debug,dump_ind);
 	      break;
-	    case 3:
-	      res=search_type_16(disk_car,partition,debug,dump_ind);
-	      break;
-	    case 4:
+	    case 2:
 	      res=search_type_2(disk_car,partition,debug,dump_ind);
 	      break;
-	    case 5:
+	    case 3:
 	      res=search_type_1(disk_car,partition,debug,dump_ind);
 	      break;
+	    case 4:
+	      /* Try to catch disklabel before BSD FFS partition */
+	      res=search_type_128(disk_car,partition,debug,dump_ind);
+	      break;
+	    case 5:
+	      /* Try to catch disklabel before BSD FFS partition */
+	      res=search_type_16(disk_car,partition,debug,dump_ind);
+	      break;
 	    case 6:
 	      res=search_type_0(disk_car,partition,debug,dump_ind);
 	      sector_inc=1;
@@ -293,19 +393,24 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	      break;
 	  }
 	}
-	else
-	  if(start.sector==7 && fast_mode>0 && ((start.head<=2)||(fast_mode==2)))
+	else if(start.sector==7 && fast_mode>0 && ((start.head<=2)||(fast_mode==2)||(search_now>0)))
+	{
+	  res=search_FAT_backup(disk_car,partition,debug,dump_ind);
+	}
+	else if(start.sector==disk_car->CHS.sector && (fast_mode>0))
+	{
+	  if((start.head==disk_car->CHS.head) || (fast_mode>1) || (search_now>0))
 	  {
-	    res=search_FAT_backup(disk_car,partition,debug,dump_ind);
+	    res=search_NTFS_backup(disk_car,partition,debug,dump_ind);
 	  }
-	  else
-	    if(start.sector==disk_car->CHS.sector && fast_mode>0)
-	    {
-	      if((start.head==disk_car->CHS.head)||(fast_mode>1))
-	      {
-		res=search_NTFS_backup(disk_car,partition,debug,dump_ind);
-	      }
-	    }
+	}
+	else if(start.sector==disk_car->CHS.sector-1 && (fast_mode>0))
+	{
+	  if((start.head==disk_car->CHS.head) || (fast_mode>1) || (search_now>0))
+	  {
+	    res=search_HFS_backup(disk_car,partition,debug,dump_ind);
+	  }
+	}
 	if(res<0)
 	{
 	  if(interface!=0)
@@ -329,7 +434,7 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	    const uint64_t hd_offset=3*(EXT2_MIN_BLOCK_SIZE<<s_log_block_size)*8*(EXT2_MIN_BLOCK_SIZE<<s_log_block_size)+(s_log_block_size==0?2*DEFAULT_SECTOR_SIZE:0);
 	    if(ref_offset>=hd_offset &&
 	     offset2sector(disk_car,ref_offset-hd_offset)==1 &&  
-	     (offset2head(disk_car,ref_offset-hd_offset)<=2 || fast_mode==2))
+	     (offset2head(disk_car,ref_offset-hd_offset)<=2 || fast_mode==2 || (search_now>0 && fast_mode>0)))
 	    {
 	      unsigned char buffer[2*DEFAULT_SECTOR_SIZE];
 	      if(disk_car->read(disk_car,sizeof(buffer), &buffer, ref_offset)==0)
@@ -368,7 +473,7 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	if(res>0)
 	{
 	  aff_part_rapport(disk_car,partition);
-	  if(disk_car->arch->is_part_type_ok(partition->part_type)!=0 && partition->part_size>1 && partition->part_offset>0)
+	  if(disk_car->arch->is_part_type_ok(partition->part_type)!=0 && partition->part_size>1 && (partition->part_offset>0 || disk_car->arch==&arch_none))
 	  {
 	    uint64_t pos_fin=partition->part_offset+partition->part_size-1;
 	    if(pos_fin<=disk_car->disk_size)
@@ -382,6 +487,29 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 		   insert_new_partition can free the memory */
 		list_part=insert_new_partition(list_part,new_partition);
 	      }
+	      {
+		uint64_t next_part_offset=partition->part_offset+partition->part_size-1+1;
+		uint64_t head_size=disk_car->CHS.sector*disk_car->sector_size;
+		if(try_offset_nbr<MAX_SEARCH_LOCATION-1)
+		{
+		  try_offset[try_offset_nbr++]=next_part_offset;
+		}
+		if(try_offset_nbr<MAX_SEARCH_LOCATION-1)
+		{
+		  try_offset[try_offset_nbr++]=next_part_offset+head_size;
+		}
+		if(next_part_offset%head_size!=0)
+		{
+		  if(try_offset_nbr<MAX_SEARCH_LOCATION-1)
+		  {
+		    try_offset[try_offset_nbr++]=(next_part_offset+head_size-1)/head_size*head_size;
+		  }
+		  if(try_offset_nbr<MAX_SEARCH_LOCATION-1)
+		  {
+		    try_offset[try_offset_nbr++]=(next_part_offset+head_size-1)/head_size*head_size+head_size;
+		  }
+		}
+	      }
 	      if(interface)
 	      {
 		aff_buffer(BUFFER_SHOW,"Q");
@@ -427,6 +555,7 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
     }
   }
   FREE(partition);
+  list_part=search_part_raid(disk_car,list_part,list_part_bad,debug,dump_ind);
   if(interface!=0 && list_part_bad!=NULL)
   {
     interface_part_bad(disk_car,list_part_bad);
@@ -771,30 +900,23 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
     if(paranoid==1 && fast_mode==2)
       can_search_deeper=0;
     list_part=search_part(disk_car,paranoid,debug,dump_ind,fast_mode,1);
-    get_geometry_from_list_part(disk_car, list_part, debug);
-    align_structure(disk_car,list_part,align);
-    /* Little trick to detect Linux software Raid 1 quickly */
-    for(element=list_part;element!=NULL;element=element->next)
+    if(list_part!=NULL)
     {
-      unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
-      struct mdp_superblock_s *sb=(struct mdp_superblock_s*)&buffer;
-      uint64_t offset=MD_NEW_SIZE_SECTORS(element->part->part_size/disk_car->sector_size)*disk_car->sector_size;
-      if(disk_car->read(disk_car,sizeof(buffer), &buffer, element->part->part_offset+offset)==0)
+      unsigned int head_max;
+      head_max=get_geometry_from_list_part(disk_car, list_part, debug);
+      if(disk_car->CHS.head!=head_max)
       {
-	t_partition *new_partition=partition_new();
-	dup_t_partition(new_partition,element->part);
-	if(test_MD(disk_car,sb,new_partition,debug,dump_ind)==0)
-	{
-	  set_MD_info(disk_car,sb,new_partition,debug,dump_ind);
-	  new_partition->part_type=P_RAID;
-	  new_partition->part_size=(uint64_t)(sb->size<<1)*disk_car->sector_size+MD_RESERVED_BYTES;
-	  list_part=insert_new_partition(list_part,new_partition);
-	}
-	else
-	  FREE(new_partition);
+	warning_geometry(disk_car,head_max+1);
       }
     }
     align_structure(disk_car,list_part,align);
+#ifdef DEBUG
+    check_list_part(list_part);
+#endif
+    align_structure(disk_car,list_part,align);
+#ifdef DEBUG
+    check_list_part(list_part);
+#endif
     disk_car->arch->init_structure(disk_car,list_part,debug);
 #ifdef DEBUG
     check_list_part(list_part);
@@ -808,11 +930,17 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
     }
     wdoprintf(stdscr,msg_GOD_MODE);
     list_part=ask_structure(disk_car,list_part,debug);
+#ifdef DEBUG
+    check_list_part(list_part);
+#endif
     if(disk_car->arch->test_structure(list_part)==0)
     {
       int do_again=0;
       int max_ext=0;
       int can_ask_minmax_ext=0;
+#ifdef DEBUG
+      check_list_part(list_part);
+#endif
       list_part=reduce_structure(list_part);
 #ifdef DEBUG
       check_list_part(list_part);
@@ -892,6 +1020,10 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
       }
       while(do_again==1);
     }
+    else
+    {
+      display_message("Invalid partition structure");
+    }
     delete_list_part(list_part);
   } while(res_interface_write=='S');
   return 0;
