@@ -2,7 +2,7 @@
 
     File: hdaccess.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,11 +37,14 @@
 #endif
 #include <errno.h>
 #include "types.h"
+#ifdef HAVE_LINUX_TYPES_H
+#include <linux/types.h>
+#endif
 #include "common.h"
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
-#ifdef HAVE_STS_DISLABEL_H
+#ifdef HAVE_SYS_DISKLABEL_H
 #include <sys/disklabel.h>
 #endif
 #ifdef HAVE_SYS_PARAM_H
@@ -65,6 +68,16 @@
 #ifdef HAVE_LINUX_FS_H
 #include <linux/fs.h>
 #endif
+#ifdef HAVE_WINDEF_H
+#include <windef.h>
+#endif
+#ifdef HAVE_WINBASE_H
+#include <stdarg.h>
+#include <winbase.h>
+#endif
+#ifdef HAVE_WINIOCTL_H
+#include <winioctl.h>
+#endif
 
 #ifdef DJGPP
 #ifdef HAVE_STDLIB_H
@@ -80,7 +93,6 @@
 #include <winioctl.h>
 #endif
 #include "fnctdsk.h"
-#include "hdaccess.h"
 #ifndef O_BINARY
 #define O_BINARY 0
 #endif
@@ -109,11 +121,13 @@ static void bug(void);
 #define MAX_IO_NBR 3
 #define MAX_HD_ERR 100
 #define PARA_CMD         8
+static void free_dos_buffer(void);
+static int alloc_dos_buffer(void);
 static int hd_super(int cmd, t_param_disk *disk_car, unsigned long int hd_offset, int nsects, void *buffer);
 static t_param_disk *hd_identify(const int debug, const unsigned int disk, const t_arch_fnct *arch);
 static int hd_identify_enh_bios(t_param_disk *param_disk,const int debug);
 static int check_enh_bios(const unsigned int disk, const int debug);
-static int hd_report_error(t_param_disk *disk_car, const unsigned long int hd_offset, const unsigned int nbr_sector, const int rc);
+static int hd_report_error(t_param_disk *disk_car, const uint64_t hd_offset, const unsigned int nbr_sector, const int rc);
 static const char *disk_description(t_param_disk *disk_car);
 static int disk_read(t_param_disk *disk_car, const unsigned int count, void *nom_buffer, const uint64_t hd_offset);
 static int disk_write(t_param_disk *disk_car, const unsigned int count, const void *nom_buffer, const uint64_t hd_offset);
@@ -129,13 +143,13 @@ struct info_disk
 static int dos_segment = 0;
 static int dos_selector = 0;
 
-void free_dos_buffer(void)
+static void free_dos_buffer(void)
 {
   __dpmi_free_dos_memory(dos_selector);
   dos_segment = dos_selector = 0;
 }
 
-int alloc_dos_buffer(void)
+static int alloc_dos_buffer(void)
 {
   if (dos_segment)
     return 0;
@@ -482,7 +496,7 @@ static int disk_read(t_param_disk *disk_car,const unsigned int count, void *nom_
       if(rc!=0)
       {
 	ecrit_rapport("disk_read");
-	hd_report_error(disk_car,hd_offset/disk_car->sector_size,count/disk_car->sector_size,rc);
+	hd_report_error(disk_car,hd_offset,count/disk_car->sector_size,rc);
       }
       return rc;
     }
@@ -509,17 +523,17 @@ static int disk_write(t_param_disk *disk_car,const unsigned int count, const voi
     if(rc)
     {
       ecrit_rapport("disk_write");
-      hd_report_error(disk_car,hd_offset/disk_car->sector_size,count/disk_car->sector_size,rc);
+      hd_report_error(disk_car,hd_offset,count/disk_car->sector_size,rc);
     }
     return rc;
   }
 }
 
-static int hd_report_error(t_param_disk *disk_car, const unsigned long int hd_offset, const unsigned int nbr_sector, const int rc)
+static int hd_report_error(t_param_disk *disk_car, const uint64_t hd_offset, const unsigned int nbr_sector, const int rc)
 {
   struct info_disk*data=disk_car->data;
   data->nbr_err++;
-  ecrit_rapport(" lba=%lu(%u/%u/%u) nbr_sector=%u, rc=%d\n",hd_offset,
+  ecrit_rapport(" lba=%lu(%u/%u/%u) nbr_sector=%u, rc=%d\n",(long unsigned int)(hd_offset/disk_car->sector_size),
       offset2cylinder(disk_car,hd_offset),offset2head(disk_car,hd_offset),offset2sector(disk_car,hd_offset),
       nbr_sector,rc);
   switch(rc)
@@ -588,6 +602,14 @@ t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct
       list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,debug,arch));
     }
   }
+#elif defined(__MINGW32__)
+    char device_hd[]="\\\\.\\PhysicalDrive0";
+    /* Disk */
+    for(i=0;i<8;i++)
+    {
+      device_hd[strlen(device_hd)-1]='0'+i;
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_hd,debug,arch));
+    }
 #elif defined(LINUX)
   {
     int j;
@@ -667,7 +689,10 @@ t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct
     /* Need to check http://mattriffle.com/mirrors/freebsd/doc/en_US.ISO8859-1/books/handbook/disks-naming.html#DISK-NAMING-PHYSICAL-TABLE */
     char device_ide[]= "/dev/rwd0";	/* raw winchester disk */
     char device_ide2[]="/dev/rad0";
+    char device_ide3[]="/dev/wd0d";    	/* NetBSD 1.6.2 IDE */
+    char device_ide4[]="/dev/rwd0c";    /* OpenBSD 3.5 IDE */
     char device_scsi[]="/dev/rda0";	/* raw scsci disk */
+    char device_scsi2[]="/dev/rsd0c";	/* OpenBSD 3.5 SCSI */
     char device_optdisk[]="/dev/rod0";
     char device_ide_hd[]="/dev/ad0";
     char device_scsi_hd[]="/dev/da0";
@@ -684,6 +709,16 @@ t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct
       list_disk=insert_new_disk(list_disk,file_test_availability(device_ide2,debug,arch));
     }
     for(i=0;i<8;i++)
+    {
+      device_ide3[strlen(device_ide3)-2]='0'+i;
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide3,debug,arch));
+    }
+    for(i=0;i<8;i++)
+    {
+      device_ide4[strlen(device_ide4)-2]='0'+i;
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide4,debug,arch));
+    }
+    for(i=0;i<8;i++)
     {
       device_ide_hd[strlen(device_ide_hd)-1]='0'+i;
       list_disk=insert_new_disk(list_disk,file_test_availability(device_ide_hd,debug,arch));
@@ -695,6 +730,11 @@ t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct
       list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,debug,arch));
     }
     for(i=0;i<8;i++)
+    {
+      device_scsi2[strlen(device_scsi2)-1]='0'+i;
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi2,debug,arch));
+    }
+    for(i=0;i<8;i++)
     {
       device_scsi_hd[strlen(device_scsi_hd)-1]='0'+i;
       list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi_hd,debug,arch));
@@ -718,7 +758,7 @@ static t_param_disk *disk_get_geometry(const int hd_h, const char *device, const
   unsigned int sector_size=0;
 #ifdef BLKSSZGET
   {
-    int arg;
+    int arg=0;
     if (ioctl(hd_h, BLKSSZGET, &arg) == 0)
     {
       sector_size=arg;
@@ -731,7 +771,7 @@ static t_param_disk *disk_get_geometry(const int hd_h, const char *device, const
 #endif
 #ifdef DIOCGSECTORSIZE
   {
-    unsigned int arg;
+    unsigned int arg=0;
     if(ioctl(hd_h,DIOCGSECTORSIZE,&arg)==0)
     {
       sector_size=arg;
@@ -810,7 +850,7 @@ static t_param_disk *disk_get_geometry(const int hd_h, const char *device, const
     if (ioctl(hd_h, BLKGETSIZE64, &longsectors64)>=0)
     {
       /* Handle a strange bug */
-      if(device>1)
+      if(debug>1)
       {
 	ecrit_rapport("disk_get_geometry BLKGETSIZE64 %s size %llu\n", device, (long long unsigned)longsectors64);
       }
@@ -853,13 +893,17 @@ static t_param_disk *disk_get_geometry(const int hd_h, const char *device, const
 #endif
     }
   }
-#ifdef __CYGWIN__
+#if defined(__CYGWIN__) || defined(__MINGW32__)
   if(disk_car==NULL)
   {
     DISK_GEOMETRY geometry;
     DWORD gotbytes;
     HANDLE handle;
+#if defined(__CYGWIN__)
     handle=(HANDLE)get_osfhandle(hd_h);
+#else
+    handle=(HANDLE)_get_osfhandle(hd_h);
+#endif
     if (DeviceIoControl( handle
 	  , IOCTL_DISK_GET_DRIVE_GEOMETRY
 	  , NULL
@@ -877,6 +921,9 @@ static t_param_disk *disk_get_geometry(const int hd_h, const char *device, const
     }
     else
     {
+#ifdef __MINGW32__
+      ecrit_rapport("DeviceIoControl failed\n");
+#else
       LPVOID buf;
       FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM
 	  , NULL
@@ -886,8 +933,9 @@ static t_param_disk *disk_get_geometry(const int hd_h, const char *device, const
 	  , 0
 	  , NULL 
 	  );
-      ecrit_rapport("DeviceIoControl failed: %s", (const char*)buf);
+      ecrit_rapport("DeviceIoControl failed: %s\n", (const char*)buf);
       LocalFree(buf);
+#endif
     }
   }
 #endif
@@ -1025,12 +1073,17 @@ static int file_read(t_param_disk *disk_car,const unsigned int count, void *nom_
 {
   if(count%disk_car->sector_size!=0 || offset%disk_car->sector_size!=0)
   {
-    char*buffer=(char*)MALLOC((count/disk_car->sector_size+2)*disk_car->sector_size);
-    /*  ecrit_rapport("file_read handle unaligned data\n"); */
+    char*buffer=(char*)MALLOC((offset%disk_car->sector_size+count+disk_car->sector_size-1)
+	/ disk_car->sector_size * disk_car->sector_size);
+    /*
+    ecrit_rapport("file_read handle unaligned data\n");
+     */
 #ifdef DEBUG
     bug();
 #endif
-    if(file_read(disk_car,(unsigned)(((count/disk_car->sector_size)+2)*disk_car->sector_size),buffer,offset/disk_car->sector_size*disk_car->sector_size))
+    if(file_read(disk_car,(offset%disk_car->sector_size+count+disk_car->sector_size-1)
+      / disk_car->sector_size * disk_car->sector_size, buffer,
+      offset/disk_car->sector_size*disk_car->sector_size))
     {
       FREE(buffer);
       return -1;
@@ -1042,18 +1095,63 @@ static int file_read(t_param_disk *disk_car,const unsigned int count, void *nom_
   {
     struct info_file *data=disk_car->data;
     long int ret;
-    if(lseek(data->handle,offset+data->offset,SEEK_SET)==-1)
+#ifdef DEBUG
+      ecrit_rapport("file_read(%d,%u,buffer,%lu(%u/%u/%u))\n", data->handle,
+	  (unsigned)(count/disk_car->sector_size), (long unsigned int)(offset/disk_car->sector_size),
+	  offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset));
+#endif
+#ifdef __MINGW32__
+    if(_lseeki64(data->handle,offset+data->offset,SEEK_SET)==-1)
     {
-      ecrit_rapport("file_read(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n", data->handle,(unsigned)(count/disk_car->sector_size),(long unsigned int)(offset/disk_car->sector_size),
-	  offset2cylinder(disk_car,offset),offset2head(disk_car,offset),offset2sector(disk_car,offset),strerror(errno));
+      ecrit_rapport("file_read(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n", data->handle,
+	  (unsigned)(count/disk_car->sector_size), (long unsigned int)(offset/disk_car->sector_size),
+	  offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset),
+	  strerror(errno));
       return -1;
     }
+#else
+    if(lseek(data->handle,offset+data->offset,SEEK_SET)==-1)
+    {
+      ecrit_rapport("file_read(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n", data->handle,
+	  (unsigned)(count/disk_car->sector_size), (long unsigned int)(offset/disk_car->sector_size),
+	  offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset),
+	  strerror(errno));
+      return -1;
+    }
+#endif
+#if defined(__CYGWIN__)
+    {
+/* November 28, 2004, CGR: cygwin read function is about 10 times slower
+   because it reads 60k each time, so lets call ReadFile directly */
+      DWORD dwByteRead;
+      HANDLE handle=(HANDLE)get_osfhandle(data->handle);
+      ret=ReadFile(handle, nom_buffer,count,&dwByteRead,NULL);
+      if(ret)
+	ret=dwByteRead;
+    }
+#else
     ret=read(data->handle, nom_buffer, count);
+#endif
     if(ret!=count)
     {
-      ecrit_rapport("file_read(%d,%u,buffer,%lu(%u/%u/%u)) read err %s (ret!=count/disk_car->sector_size*sector_size)\n", data->handle,(unsigned)(count/disk_car->sector_size),(long unsigned)(offset/disk_car->sector_size),
-	  offset2cylinder(disk_car,offset),offset2head(disk_car,offset),offset2sector(disk_car,offset),(ret==-1?strerror(errno):"Truncated file"));
-      return -1;
+      ecrit_rapport("file_read(%d,%u,buffer,%lu(%u/%u/%u)) read err: ", data->handle,
+	  (unsigned)(count/disk_car->sector_size), (long unsigned)(offset/disk_car->sector_size),
+	  offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset));
+      if(ret<0)
+      {
+	ecrit_rapport("%s\n", strerror(errno));
+	return -1;
+      }
+      else if(ret==0)
+      {
+	ecrit_rapport("read after end of file\n");
+	return -1;
+      }
+      else
+      {
+	ecrit_rapport("Partial read\n");
+	memset((char*)nom_buffer+ret,0,count-ret);
+      }
     }
   }
   return 0;
@@ -1083,18 +1181,27 @@ static int file_write(t_param_disk *disk_car,const unsigned int count, const voi
     long int ret;
     struct info_file *data=disk_car->data;
 /*    ecrit_rapport("file_write offset=%lu data->offset=%lu\n",(long unsigned)offset,(long unsigned)data->offset); */
+#ifdef __MINGW32__
+    if(_lseeki64(data->handle,offset+data->offset,SEEK_SET)==-1)
+    {
+      ecrit_rapport("file_write(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n", data->handle,(unsigned)(count/disk_car->sector_size),(long unsigned)(offset/disk_car->sector_size),
+	  offset2cylinder(disk_car,offset),offset2head(disk_car,offset),offset2sector(disk_car,offset),strerror(errno));
+      return -1;
+    }
+#else
     if(lseek(data->handle,offset+data->offset,SEEK_SET)==-1)
     {
       ecrit_rapport("file_write(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n", data->handle,(unsigned)(count/disk_car->sector_size),(long unsigned)(offset/disk_car->sector_size),
 	  offset2cylinder(disk_car,offset),offset2head(disk_car,offset),offset2sector(disk_car,offset),strerror(errno));
       return -1;
     }
+#endif
     disk_car->write_used=1;
     ret=write(data->handle, nom_buffer, count);
     if(ret!=count)
     {
       ecrit_rapport("file_write(%d,%u,buffer,%lu(%u/%u/%u)) write err %s\n", data->handle,(unsigned)(count/disk_car->sector_size),(long unsigned)(offset/disk_car->sector_size),
-	  offset2cylinder(disk_car,offset),offset2head(disk_car,offset),offset2sector(disk_car,offset),(ret==-1?strerror(errno):"File truncated"));
+	  offset2cylinder(disk_car,offset),offset2head(disk_car,offset),offset2sector(disk_car,offset),(ret<0?strerror(errno):"File truncated"));
       return -1;
     }
     return 0;
@@ -1176,8 +1283,13 @@ t_param_disk *file_test_availability(const char *device, const int debug, const
       {
 	ecrit_rapport("file_test_availability %s: fstat failed\n",device);
       }
+#ifdef __MINGW32__
+      stat_rec.st_mode=S_IFBLK;
+      stat_rec.st_size=0;
+#else
       close(hd_h);
       return NULL;
+#endif
     }
 #ifndef DJGPP
     if(!S_ISREG(stat_rec.st_mode))
@@ -1197,7 +1309,7 @@ t_param_disk *file_test_availability(const char *device, const int debug, const
     if(disk_car==NULL || disk_car->disk_size==0)
     {
       unsigned char buffer[DEFAULT_SECTOR_SIZE];
-      if(read(hd_h,buffer,DEFAULT_SECTOR_SIZE)!=DEFAULT_SECTOR_SIZE)
+      if(read(hd_h,buffer,DEFAULT_SECTOR_SIZE)<0)
       {
 	close(hd_h);
 	return NULL;
@@ -1235,7 +1347,7 @@ t_param_disk *file_test_availability(const char *device, const int debug, const
 	}
 	if(stat_rec.st_size>offset)
 	{
-	  disk_car->disk_size=(uint64_t)(stat_rec.st_size-offset)/disk_car->sector_size*disk_car->sector_size;
+	  disk_car->disk_size=(uint64_t)(stat_rec.st_size-offset+disk_car->sector_size-1)/disk_car->sector_size*disk_car->sector_size;
 	}
 	else
 	{
@@ -1262,6 +1374,7 @@ t_param_disk *file_test_availability(const char *device, const int debug, const
       data=MALLOC(sizeof(*data));
       data->offset=offset;
       strncpy(data->name,device,sizeof(data->name));
+      data->name[sizeof(data->name)-1]='\0';
       data->handle=hd_h;
       disk_car->halt_on_errors=0;
       disk_car->write_used=0;
@@ -1285,7 +1398,7 @@ void hd_parse_bis(const t_list_disk * list_disk, const int allow_partial_last_cy
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
   {
     t_param_disk *disk_car=element_disk->disk;
-    char buffer[disk_car->sector_size];
+    unsigned char buffer[disk_car->sector_size];
     uint64_t pos;
     t_CHS pos_CHS;
     if(disk_car->autodetect!=0)
