@@ -2,7 +2,7 @@
 
     File: intrface.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #endif
  
 #include <stdarg.h>
-#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -32,9 +31,6 @@
 #include <string.h>
 #endif
 #include <ctype.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #ifdef HAVE_TIME_H
 #include <time.h>
 #endif
@@ -62,6 +58,7 @@
 
 extern const t_arch_fnct arch_i386;
 extern const t_arch_fnct arch_sun;
+extern const t_arch_fnct arch_none;
 
 static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *align,int *allow_partial_last_cylinder,int *ask_part_order,int *halt_on_errors, unsigned int *expert, const t_arch_fnct **arch);
 static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part, const int debug);
@@ -99,7 +96,7 @@ void interface_list(t_param_disk *disk_car, const int debug,const int test_recov
 static int write_MBR_code(t_param_disk *disk_car)
 {
   wdoprintf(stdscr,msg_WRITE_MBR_CODE);
-  if(ask_YN(stdscr)!=0 && ask_confirmation("Write a new copy of MBR code, confirm ? (Y/N)")!=0)
+  if(ask_YN(stdscr)!=0 && ask_confirmation("Write a new copy of i386 MBR code, confirm ? (Y/N)")!=0)
   {
     unsigned char buffer[DEFAULT_SECTOR_SIZE];
     if(read_MBR(disk_car,&buffer)!=0)
@@ -167,18 +164,21 @@ int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, con
     setenv("TERM","cygwin",0);
 #endif
 #endif
-#ifdef DJGPP
-  initscr();
+#if defined(DJGPP) || defined(__MINGW32__)
+  if(initscr()==NULL)
+  {
+    printf("initscr() has failed. Exiting\n");
+  }
 #else
   if(newterm(NULL,stdout,stdin)==NULL)
   {
 #ifdef HAVE_SETENV
     setenv("TERMINFO",".",1);
-#endif
     if(newterm(NULL,stdout,stdin)==NULL)
+#endif
     {
-      ecrit_rapport("Terminfo file is missing\n");
-      printf("Terminfo file is missing\n");
+      ecrit_rapport("Terminfo file is missing.\n");
+      printf("Terminfo file is missing.\nCheck if you have not forgot to extract the subdirectories.\n");
       return 1;
     }
   }
@@ -187,6 +187,7 @@ int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, con
   {
     ecrit_rapport("Terminal has only %u lines\n",LINES);
     endwin();
+    printf("TestDisk need 25 lines to work.\nPlease enlarge the terminal and restart TestDisk.\n");
     return 1;
   }
   noecho();
@@ -298,6 +299,7 @@ void interface_analyse(t_param_disk *disk_car, const int debug, const int savehe
   /* log */
   ecrit_rapport("\nAnalyse ");
   ecrit_rapport("%s\n",disk_car->description(disk_car));
+  aff_buffer(BUFFER_RESET,"Q");
   /* ncurses interface */
   aff_copy(stdscr);
   mvwaddstr(stdscr,21,0,msg_PART_TYPE);
@@ -320,11 +322,11 @@ int interface_write(t_param_disk *disk_car,t_list_part *list_part,const int can_
 {
   t_list_part *parts;
   ecrit_rapport("\ninterface_write()\n");
+  aff_buffer(BUFFER_RESET,"Q");
   aff_copy(stdscr);
   wmove(stdscr,4,0);
   wdoprintf(stdscr,"%s",disk_car->description(disk_car));
   wmove(stdscr,5,0);
-  aff_buffer(BUFFER_RESET,"Q");
   mvwaddstr(stdscr,6,0,msg_PART_HEADER_LONG);
   for(parts=list_part;parts!=NULL;parts=parts->next)
     if(parts->part->status!=STATUS_LOG)
@@ -333,7 +335,10 @@ int interface_write(t_param_disk *disk_car,t_list_part *list_part,const int can_
     if(parts->part->status==STATUS_LOG)
       aff_part_buffer(AFF_PART_ORDER,disk_car,parts->part);
   if(list_part==NULL)
+  {
+    aff_buffer(BUFFER_ADD," \nNo partition found or selected for recovery");
     return aff_buffer(BUFFER_DISPLAY,can_search_deeper?"S":"",stdscr);
+  }
   else
     return aff_buffer(BUFFER_DISPLAY,(can_search_deeper?(can_ask_minmax_ext?"SWE":"SW"):(can_ask_minmax_ext?"WE":"W")),stdscr);
 }
@@ -395,7 +400,7 @@ static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *al
     menuOptions[6].name=*halt_on_errors?"Halt on errors : Yes":"Halt on errors : No";
     menuOptions[7].name=*expert?"Expert mode : Yes":"Expert mode : No";
     menuOptions[8].name=(*arch)->part_name;
-    /*	car=toupper( wmenuSelect(stdscr,INTER_OPTION_Y, INTER_OPTION_X, menuOptions, 0, "PDCFAOEMQ", MENU_VERT, menu)); */
+    aff_copy(stdscr);
     car=toupper( wmenuSelect(stdscr,INTER_OPTION_Y, INTER_OPTION_X, menuOptions, 0, "DCAOHMETQ", MENU_VERT, menu));
     switch(car)
     {
@@ -440,6 +445,8 @@ static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *al
       case 'T':
        if(*arch==&arch_i386)
 	 *arch=&arch_sun;
+       else if(*arch==&arch_sun)
+	 *arch=&arch_none;
        else
 	 *arch=&arch_i386;
 	menu=8;
@@ -484,7 +491,7 @@ static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *al
       *ask_part_order?"Yes":"No",
       *halt_on_errors?"Yes":"No",
       *expert?"Yes":"No",
-      (*arch==&arch_i386?"Intel":"Sun"));
+      (*arch==&arch_i386?"Intel":(*arch==&arch_sun?"Sun":"None")));
 }
 
 t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const int debug)
@@ -547,7 +554,10 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
     }
     else
     {
-      waddstr(stdscr,"T: change type, ");
+      if(pos->part->arch==NULL || pos->part->arch==disk_car->arch)
+      {
+	waddstr(stdscr,"T: change type, ");
+      }
       switch(pos->part->upart_type)
       {
 	case UP_EXT2:
@@ -637,14 +647,16 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
       case ' ':
 	if(list_part!=NULL)
 	{
-	  disk_car->arch->set_next_status(disk_car,pos->part);
+	  if(pos->part->arch==NULL || pos->part->arch==disk_car->arch)
+	    disk_car->arch->set_next_status(disk_car,pos->part);
 	}
 	break;
       case KEY_LEFT:
       case '-':
 	if(list_part!=NULL)
 	{
-	  disk_car->arch->set_prev_status(disk_car,pos->part);
+	  if(pos->part->arch==NULL || pos->part->arch==disk_car->arch)
+	    disk_car->arch->set_prev_status(disk_car,pos->part);
 	}
 	break;
       case 'a':
@@ -661,8 +673,11 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
       case 'T':
 	if(list_part!=NULL)
 	{
-	  rewrite=1;
-	  change_part_type(disk_car,pos->part);
+	  if(pos->part->arch==NULL || pos->part->arch==disk_car->arch)
+	  {
+	    rewrite=1;
+	    change_part_type(disk_car,pos->part);
+	  }
 	}
 	break;
       case 'p':
@@ -695,10 +710,9 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
 	      break;
 	  }
 	  delwin(window);
-#ifdef DJGPP
-	  wredrawln(stdscr,0,stdscr->_maxy);	/* redrawwin def is boggus in pdcur24 */
-#else
-	  redrawwin(stdscr);	/* stdscr has been corrupted by window */
+	  (void) clearok(stdscr, TRUE);
+#ifdef HAVE_TOUCHWIN
+	  touchwin(stdscr);
 #endif
 	}
 	break;
@@ -768,34 +782,49 @@ static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part
       {
 	aff_copy(stdscr);
 	mvwaddstr(stdscr,4,0,disk_car->description(disk_car));
-	mvwaddstr(stdscr,5,0,"Choose the backup you want to restore:");
-	mvwaddstr(stdscr,20,0,"PS: Don't worry you will have to confirm the partition restoration.");
+	if(list_backup!=NULL)
+	{
+	  mvwaddstr(stdscr,5,0,"Choose the backup you want to restore:");
+	  mvwaddstr(stdscr,20,0,"PS: Don't worry you will have to confirm the partition restoration.");
+	}
+	else
+	{
+	  mvwaddstr(stdscr,5,0,"No backup found!");
+	}
 	rewrite=0;
       }
-      for(i=0,backup=list_backup;(backup!=NULL) && (i<offset);backup=backup->next,i++);
-      for(i=offset;(backup!=NULL) &&((i-offset)<INTER_STRUCTURE);i++,backup=backup->next)
+      if(list_backup!=NULL)
       {
-	wmove(stdscr,8+i-offset,0);
-	wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
-	if(backup==pos)
+	for(i=0,backup=list_backup;(backup!=NULL) && (i<offset);backup=backup->next,i++);
+	for(i=offset;(backup!=NULL) &&((i-offset)<INTER_STRUCTURE);i++,backup=backup->next)
 	{
-	  wattrset(stdscr,A_STANDOUT);
-	  wdoprintf(stdscr,"%s %s",backup->description,ctime(&backup->my_time));
-	  wattroff(stdscr,A_STANDOUT);
-	} else
+	  wmove(stdscr,8+i-offset,0);
+	  wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
+	  if(backup==pos)
+	  {
+	    wattrset(stdscr,A_STANDOUT);
+	    wdoprintf(stdscr,"%s %s",backup->description,ctime(&backup->my_time));
+	    wattroff(stdscr,A_STANDOUT);
+	  } else
+	  {
+	    wdoprintf(stdscr,"%s %s",backup->description,ctime(&backup->my_time));
+	  }
+	}
+	if(i<=INTER_STRUCTURE && backup==NULL)
 	{
-	  wdoprintf(stdscr,"%s %s",backup->description,ctime(&backup->my_time));
+	  strncpy(options,"LQ",sizeof(options));
+	  menu=0;
+	}
+	else
+	{
+	  strncpy(options,"PNLQ",sizeof(options));
+	  menu=2;
 	}
-      }
-      if(i<=INTER_STRUCTURE && backup==NULL)
-      {
-	strncpy(options,"LQ",sizeof(options));
-	menu=0;
       }
       else
       {
-	strncpy(options,"PNLQ",sizeof(options));
-	menu=2;
+	menu=0;
+	strncpy(options,"Q",sizeof(options));
       }
       switch(wmenuSelect(stdscr,INTER_DUMP_Y,INTER_DUMP_X, menuLoadBackup, 8, options, MENU_HORIZ| MENU_BUTTON | MENU_ACCEPT_OTHERS, menu))
       {
@@ -884,3 +913,28 @@ static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part
   }
   return list_part;
 }
+
+int interface_superblock(t_param_disk *disk_car,t_list_part *list_part)
+{
+  t_list_part *parts;
+  t_partition *old_part=NULL;
+  aff_buffer(BUFFER_RESET,"Q");
+  aff_copy(stdscr);
+  wmove(stdscr,4,0);
+  wdoprintf(stdscr,"%s",disk_car->description(disk_car));
+  wmove(stdscr,5,0);
+  mvwaddstr(stdscr,6,0,msg_PART_HEADER_LONG);
+  for(parts=list_part;parts!=NULL;parts=parts->next)
+  {
+    if(old_part==NULL || (old_part->part_offset!=parts->part->part_offset) ||
+      old_part->part_size!=parts->part->part_size ||
+      old_part->part_type!=parts->part->part_type ||
+      old_part->upart_type!=parts->part->upart_type)
+    {
+      aff_part_buffer(AFF_PART_SHORT,disk_car,parts->part);
+      old_part=parts->part;
+    }
+    aff_buffer(BUFFER_ADD,"superblock %u, blocksize=%u\n",parts->part->boot_sector,parts->part->blocksize);
+  }
+  return aff_buffer(BUFFER_DISPLAY,"",stdscr);
+}
