@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
  
-#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -77,8 +76,8 @@ int recover_JFS(t_param_disk *disk_car, const struct jfs_superblock *sb,t_partit
     ecrit_rapport("\n");
     ecrit_rapport("recover_JFS: s_blocksize=%lu\n",partition->blocksize);
     ecrit_rapport("recover_JFS: s_size %lu\n",(long unsigned int)le64(sb->s_size));
-    ecrit_rapport("recover_JFS: s_fsckpxd.len:%d\n", le24(sb->s_fsckpxd.len));
-    ecrit_rapport("recover_JFS: s_logpxd.len:%d\n", le24(sb->s_logpxd.len));
+    ecrit_rapport("recover_JFS: s_fsckpxd.len:%d\n", (int)le24(sb->s_fsckpxd.len));
+    ecrit_rapport("recover_JFS: s_logpxd.len:%d\n", (int)le24(sb->s_logpxd.len));
     ecrit_rapport("recover_JFS: part_size %lu\n",(long unsigned)(partition->part_size/disk_car->sector_size));
   }
   return 0;
