@@ -17,8 +17,8 @@
  *   along with this program;  if not, write to the Free Software 
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
-#ifndef	_H_JFS_SUPERBLOCK
-#define _H_JFS_SUPERBLOCK
+#ifndef	_JFS_SUPERBLOCK_H
+#define _JFS_SUPERBLOCK_H
 
 /*
  * make the magic number something a human could read
@@ -123,4 +123,4 @@ struct jfs_superblock {
 
 };
 
-#endif /*_H_JFS_SUPERBLOCK */
+#endif /*_JFS_SUPERBLOCK_H */
