@@ -21,11 +21,10 @@
  */
 
 #define msg_DUMP_HEXA     "Dump Hexa\n"
-#define msg_READ_ERROR_AT "\nRead error at %u/%u/%u (lba=%lu)"
+#define msg_READ_ERROR_AT "Read error at %u/%u/%u (lba=%lu)\n"
 #define c_YES           'Y'
 #define c_NO            'N'
-#define msg_TBL_NMARK     "\nPartition sector doesn't have the endmark 0xAA55\nRun \"MBR code\" in TestDisk to correct it\n"
-#define msg_TBL2_NMARK     "\nPartition sector doesn't have the endmark 0xAA55\n"
+#define msg_TBL_NMARK     "\nPartition sector doesn't have the endmark 0xAA55\n"
 #define msg_BAD_NMARK     "\nBoot sector doesn't have the endmark 0xAA55\n"
 #define msg_BAD_S_CYL     "\nBad starting cylinder"
 #define msg_BAD_S_HEAD    "\nBad starting head"
@@ -41,7 +40,6 @@
 #define msg_CHKFAT_SECT_CLUSTER   "check_FAT: Bad number of sectors per cluster\n"
 #define msg_CHKFAT_SECTTRACK      "check_FAT: Incorrect number of sectors per track\n"
 #define msg_CHKFAT_SIDE   "check_FAT: Incorrect number of sides\n"
-#define msg_CHKFAT_NBRFAT "check_FAT: Bad number of FAT\n"
 #define msg_CHKFAT_ENTRY  "check_FAT: Bad number of entries in root dir\n"
 #define msg_CHKFAT_SIZE   "check_FAT: Incorrect size of partition\n"
 #define msg_CHKFAT_SECTPFAT "check_FAT: Incorrect number of sectors per FAT\n"
@@ -58,7 +56,6 @@
 #define msg_NO_BOOTABLE   "No partition is bootable\n"
 #define msg_ONLY1MUSTBOOT "Only one partition must be bootable\n"
 #define msg_SAME_SPACE    "Space conflict between the following two partitions\n"
-#define msg_DISK		"Disk"
 #define msg_CHOICE        "\nChoice ? "
 #define msg_MALLOC_ERROR  "\nMemory allocation error"
 #define msg_FREE_ERROR    "\nFree memory allocation error"
@@ -72,5 +69,5 @@
 #define msg_NO_EXT_PART   "\nNo extended partition"
 #define msg_ROOT_CLUSTER_RERR	"\nroot_cluster: read error"
 #define msg_PART_TYPE "*=Primary bootable  P=Primary  L=Logical  E=Extended  D=Deleted"
-const char *partition_type(const unsigned char type, const struct systypes *parttype_name_table);
+const unsigned char *partition_type(const unsigned char type, const struct systypes *parttype_name_table);
 
