@@ -19,8 +19,8 @@
  *
  */
 
-#ifndef _GM_HMLVM_H
-#define _GM_HMLVM_H
+#ifndef _LVM_H
+#define _LVM_H
 
 /*
  * structs & defines gathered from LVM 0.7/0.9 lvm.h and liblvm.h
@@ -90,5 +90,33 @@ int check_LVM(t_param_disk *disk_car,t_partition *partition,const int debug);
 int test_LVM(t_param_disk *disk_car, const pv_disk_t *pv,t_partition *partition,const int debug, const int dump_ind);
 int recover_LVM(t_param_disk *disk_car, const pv_disk_t *pv,t_partition *partition,const int debug, const int dump_ind);
 
+#define LVM2_LABEL	"LVM2 001"
+#define LABEL_ID 	"LABELONE"
 
-#endif /* _GM_HMLVM_H */
+struct lvm2_label_header {
+  uint8_t id[8];          /* 0x00 LABELONE */
+  uint64_t sector_xl;     /* 0x08 Sector number of this label */
+  uint32_t crc_xl;        /* 0x10 From next field to end of sector */
+  uint32_t offset_xl;     /* 0x14 Offset from start of struct to contents */
+  uint8_t type[8];        /* 0x18 LVM2 001 */
+} __attribute__ ((packed));
+
+struct lvm2_disk_locn {
+  uint64_t offset;        /* Offset in bytes to start sector */
+  uint64_t size;          /* Bytes */
+} __attribute__ ((packed));
+                                                                                                                             
+
+struct lvm2_pv_header {
+  uint8_t pv_uuid[32];
+  uint64_t device_size_xl;        /* Bytes */
+  /* NULL-terminated list of data areas followed by */
+  /* NULL-terminated list of metadata area headers */
+  struct lvm2_disk_locn disk_areas_xl[0];      /* Two lists */
+} __attribute__ ((packed));
+
+int check_LVM2(t_param_disk *disk_car,t_partition *partition,const int debug);
+int test_LVM2(t_param_disk *disk_car, const struct lvm2_label_header *lh,t_partition *partition,const int debug, const int dump_ind);
+int recover_LVM2(t_param_disk *disk_car, const unsigned char *buf,t_partition *partition,const int debug, const int dump_ind);
+
+#endif /* _LVM_H */
