@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
  
-#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -32,10 +31,15 @@
 #include "md.h"
 #include "intrf.h"
 #include "fnctdsk.h"
+
 int check_MD(t_param_disk *disk_car,t_partition *partition,const int debug)
 {
   unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
-  uint64_t offset=MD_NEW_SIZE_SECTORS(partition->part_size/disk_car->sector_size)*disk_car->sector_size;
+  uint64_t offset=MD_NEW_SIZE_SECTORS(partition->part_size/512)*512;
+  if(debug>1)
+  {
+    ecrit_rapport("Raid offset %llu\n",(long long unsigned)offset/512);
+  }
   if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+offset)!=0)
   { return 1; }
   if(test_MD(disk_car,(struct mdp_superblock_s*)&buffer,partition,debug,0)!=0)
@@ -70,8 +74,11 @@ int recover_MD(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_parti
     return 1;
   set_MD_info(disk_car,sb,partition,debug,dump_ind);
   partition->part_type=(unsigned char)P_RAID;
-  partition->part_offset=partition->part_offset-(sb->size<<1);
   partition->part_size=(uint64_t)(sb->size<<1)*disk_car->sector_size+MD_RESERVED_BYTES;
+  partition->part_offset=partition->part_offset-partition->part_size;
+  /*
+  partition->part_offset=partition->part_offset-(partition->part_size+sb->chunk_size-1)/sb->chunk_size*sb->chunk_size;
+  */
   return 0;
 }
 
@@ -79,9 +86,13 @@ int test_MD(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_partitio
 {
   if(sb->md_magic==(unsigned int)MD_SB_MAGIC)
   {
+    ecrit_rapport("\nRaid magic value at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
+    ecrit_rapport("Raid apparent size: %llu sectors\n",(long long unsigned)(sb->size<<1));
+    ecrit_rapport("Raid chunk size: %llu bytes\n",(long long unsigned)sb->chunk_size);
+    if(sb->chunk_size==0)
+      return 1;
     if(dump_ind!=0)
     {
-      ecrit_rapport("\nRaid magic value at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
       /* There is a little offset ... */
       dump(stdscr,sb,DEFAULT_SECTOR_SIZE);
     }
