@@ -28,12 +28,18 @@ struct ntfs_boot_sector {
 	uint8_t  	bootstrap[426];             	/* 0x54 Irrelevant (boot up code). */
 	uint16_t	marker;				/* 0x1FE */
 	} __attribute__ ((__packed__));
-int set_NTFS_info(t_param_disk *disk_car, const struct ntfs_boot_sector*ntfs_header,t_partition *partition,const int debug, const int dump_ind);
-int recover_NTFS(t_param_disk *disk_car, const struct ntfs_boot_sector*ntfs_header,t_partition *partition,const int debug, const int dump_ind, const int backup);
 int check_NTFS(t_param_disk *disk_car,t_partition *partition,const int debug,const int dump_ind);
-int test_NTFS(const t_param_disk *disk_car,const struct ntfs_boot_sector*ntfs_header, t_partition *partition,const int debug, const int dump_ind);
 int dir_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition);
-int is_ntfs(const int part_type);
+int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug);
 int dump_2ntfs_rapport(const struct ntfs_boot_sector *nh1, const struct ntfs_boot_sector *nh2);
 int dump_ntfs_rapport(const struct ntfs_boot_sector *ntfs_header);
-int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug);
+int is_ntfs(const int part_type);
+int ntfs_get_attr(const char *mft_record, const int my_type, t_partition *partition, const char *end, const int debug, const int dump_ind, const char*file_name_to_find);
+int recover_NTFS(t_param_disk *disk_car, const struct ntfs_boot_sector*ntfs_header,t_partition *partition,const int debug, const int dump_ind, const int backup);
+int set_NTFS_info(t_param_disk *disk_car, const struct ntfs_boot_sector*ntfs_header,t_partition *partition,const int debug, const int dump_ind);
+int test_NTFS(const t_param_disk *disk_car,const struct ntfs_boot_sector*ntfs_header, t_partition *partition,const int debug, const int dump_ind);
+#define NTFS_GETU8(p)      (*(const uint8_t*)(p))
+#define NTFS_GETU16(p)     (*(const uint16_t*)(p))
+#define NTFS_GETU32(p)     (*(const uint32_t*)(p))
+#define NTFS_GETU64(p)     (*(const uint64_t*)(p))
+ 
