@@ -31,7 +31,6 @@
 
 #include <sys/stat.h>
 
-#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -58,6 +57,7 @@
 #include "intrf.h"
 #include "ntfs.h"
 #include "dir.h"
+#include "io_redir.h"
 struct list_dir_struct {
 	t_file_data *dir_list;
 	t_file_data *current_file;
@@ -406,7 +406,10 @@ int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partit
   t_my_data my_data;
   static struct ntfs_device *dev;
   ntfs_volume *vol=NULL;
+  if(partition->boot_sector!=0)
+    io_redir_add_redir(disk_car,partition->part_offset,DEFAULT_SECTOR_SIZE,partition->part_offset+(uint64_t)partition->boot_sector*disk_car->sector_size,NULL);
   aff_buffer(BUFFER_RESET,"Q");
+  aff_copy(window);
   wmove(window,4,0);
   aff_part(window,AFF_PART_ORDER,disk_car,partition);
   my_data.partition=partition;
@@ -423,6 +426,8 @@ int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partit
     aff_part_rapport(disk_car,partition);
     aff_buffer(BUFFER_ADD,"Can't open NTFS filesystem: %s\n",strerror (errno));
     aff_buffer(BUFFER_DISPLAY,"Q",window);
+    if(partition->boot_sector!=0)
+      io_redir_del_redir(disk_car,partition->part_offset);
     return 2;
   }
   if (vol->flags & VOLUME_IS_DIRTY) {
@@ -442,17 +447,21 @@ int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partit
     dir_partition(disk_car,partition,&dir_data,FILE_root);
   }  
   ntfs_umount(vol, FALSE);
+  if(partition->boot_sector!=0)
+    io_redir_del_redir(disk_car,partition->part_offset);
   return 0;
 }
 #else
 #include "types.h"
 #include "common.h"
 #include "intrf.h"
+#include "ntfs.h"
 
-int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition)
+int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug)
 {
   aff_buffer(BUFFER_RESET,"Q");
-  wmove(stdscr,4,0);
+  aff_copy(window);
+  wmove(window,4,0);
   aff_part(window,AFF_PART_ORDER,disk_car,partition);
   aff_part_rapport(disk_car,partition);
   aff_buffer(BUFFER_ADD,"Recompile with ntfsprogs library");
