@@ -27,9 +27,6 @@
 #endif
  
 #ifdef HAVE_LIBNTFS
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -37,7 +34,6 @@
 #include <string.h>
 #endif
 #include <errno.h>
-#include <stdio.h>
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -45,10 +41,6 @@
 #include <fcntl.h>
 #endif
 #include <sys/ioctl.h>
-#ifdef HAVE_LINUX_FD_H
-//#	include <linux/fd.h>
-#endif
-
 #include <ntfs/device.h>
 
 #include "types.h"
@@ -57,8 +49,6 @@
 #if defined(linux) && defined(_IO) && !defined(BLKGETSIZE)
 #	define BLKGETSIZE _IO(0x12,96) /* Get device size in 512byte blocks. */
 #endif
-/*extern s64 ntfs_pread(struct ntfs_device *dev, const s64 pos, s64 count, void *b); */
-/*extern s64 ntfs_pwrite(struct ntfs_device *dev, const s64 pos, s64 count, const void *b); */
 
 static int ntfs_device_testdisk_io_open(struct ntfs_device *dev, int flags)
 {
