@@ -24,7 +24,6 @@
 #include <config.h>
 #endif
  
-#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -109,7 +108,7 @@ int recover_rfs(t_param_disk *disk_car, const struct reiserfs_super_block *sb,t_
   }
 
   partition->part_size = (uint64_t)le32(sb->s_block_count) * le16(sb->s_blocksize);
-  partition->part_type = (unsigned char)0x83;
+  partition->part_type = (unsigned char)P_LINUX;
   set_rfs_info(disk_car,sb,partition,debug,dump_ind);
   return 0;
 }
@@ -124,7 +123,7 @@ static int set_rfs_info(const t_param_disk *disk_car,const struct reiserfs_super
       break;
     case UP_RFS2:
       strncpy(partition->info,"ReiserFS 3.6 with standard journal",sizeof(partition->info));
-      set_part_name(partition,sb->s_label,16);
+      set_part_name(partition,(const char*)sb->s_label,16);
       break;
     case UP_RFS3:
       if(le16(sb->sb_version)==1)
@@ -134,7 +133,7 @@ static int set_rfs_info(const t_param_disk *disk_car,const struct reiserfs_super
 	strncpy(partition->info,"ReiserFS 3.6 with non standard journal",sizeof(partition->info));
       else
 	strncpy(partition->info,"ReiserFS ? with non standard journal",sizeof(partition->info));
-      set_part_name(partition,sb->s_label,16);
+      set_part_name(partition,(const char*)sb->s_label,16);
       break;
     default:
       return 1;
