@@ -12,27 +12,27 @@
 
 struct reiserfs_super_block
 {
-	uint32_t s_block_count;		/* blocks count         */
-	uint32_t s_free_blocks;		/* free blocks count    */
-	uint32_t s_root_block;		/* root block number    */
-	uint32_t s_journal_block;		/* journal block number    */
-	uint32_t s_journal_dev;		/* journal device number  */
-	uint32_t s_orig_journal_size;	/* size of the journal on FS creation.  used to make sure they don't overflow it */
-	uint32_t s_journal_trans_max;	/* max number of blocks in a transaction.  */
-	uint32_t s_journal_block_count;	/* total size of the journal. can change over time  */
-	uint32_t s_journal_max_batch;	/* max number of blocks to batch into a trans */
-	uint32_t s_journal_max_commit_age;	/* in seconds, how old can an async commit be */
-	uint32_t s_journal_max_trans_age;	/* in seconds, how old can a transaction be */
-	uint16_t s_blocksize;		/* block size           */
-	uint16_t s_oid_maxsize;		/* max size of object id array, see get_objectid() commentary  */
-	uint16_t s_oid_cursize;		/* current size of object id array */
-	uint16_t s_state;			/* valid or error       */
-	char s_magic[10];		/* reiserfs magic string indicates that file system is reiserfs */
-	uint16_t sb_fs_state; 		/* 62 it is set to used by fsck to mark which phase of
-				   rebuilding is done (used for fsck debugging) */
-	uint32_t s_hash_function_code;	/* indicate, what hash fuction is being use to sort names in a directory*/
-	uint16_t s_tree_height;		/* height of disk tree */
-	uint16_t s_bmap_nr;		/* amount of bitmap blocks needed to address each block of file system */
+	uint32_t s_block_count;		/* 0x00 blocks count         */
+	uint32_t s_free_blocks;		/* 0x04 free blocks count    */
+	uint32_t s_root_block;		/* 0x08 root block number    */
+	uint32_t s_journal_block;	/* 0x0C journal block number    */
+	uint32_t s_journal_dev;		/* 0x10 journal device number  */
+	uint32_t s_orig_journal_size;	/* 0x14 size of the journal on FS creation. */
+	uint32_t s_journal_trans_max;	/* 0x18 max number of blocks in a transaction.  */
+	uint32_t s_journal_block_count;	/* 0x1C total size of the journal. can change over time  */
+	uint32_t s_journal_max_batch;	/* 0x20 max number of blocks to batch into a trans */
+	uint32_t s_journal_max_commit_age;	/* 0x24 in seconds, how old can an async commit be */
+	uint32_t s_journal_max_trans_age;	/* 0x28 in seconds, how old can a transaction be */
+	uint16_t s_blocksize;		/* 0x2C block size           */
+	uint16_t s_oid_maxsize;		/* 0x2E max size of object id array, see get_objectid() commentary  */
+	uint16_t s_oid_cursize;		/* 0x30 current size of object id array */
+	uint16_t s_state;		/* 0x32 valid or error       */
+	char s_magic[10];		/* 0x34 reiserfs magic string indicates that file system is reiserfs */
+	uint16_t sb_fs_state; 		/* 0x3E it is set to used by fsck to mark which phase of
+					   rebuilding is done (used for fsck debugging) */
+	uint32_t s_hash_function_code;	/* 0x40 indicate, what hash fuction is being use to sort names in a directory*/
+	uint16_t s_tree_height;		/* 0x44 height of disk tree */
+	uint16_t s_bmap_nr;		/* 0x46 amount of bitmap blocks needed to address each block of file system */
 	uint16_t sb_version; 		/* 72 this field is only reliable on
 					   filesystem with non-standard journal */
 	uint16_t sb_reserved_for_journal;  /* 74 size in blocks of journal area on
