@@ -25,23 +25,12 @@
 #endif
  
 #ifdef HAVE_LIBREISERFS
-#include <stdio.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-#include <time.h>
 #include "dal/dal.h"
 #ifdef HAVE_DAL_FILE_DAL_H
 #include "dal/file_dal.h"
@@ -521,7 +510,7 @@ dir_partition_reiser_error:
 int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug)
 {
   aff_buffer(BUFFER_RESET,"Q");
-  wmove(stdscr,4,0);
+  wmove(window,4,0);
   aff_part(window,AFF_PART_ORDER,disk_car,partition);
   aff_part_rapport(disk_car,partition);
   aff_buffer(BUFFER_ADD,"Recompile with progsreiserfs library");
