@@ -2,7 +2,7 @@
 
     File: testdisk.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -97,8 +97,9 @@ static FILE* init_log(const char*filename,int argc, char**argv)
   FILE*f_file=fopen(filename,"a");
   if(f_file==NULL)
   {
-    printf("\nCan not create testdisk.log: %s\n",strerror(errno));
-    exit(1);
+    printf("\nWarning: Can not create testdisk.log: %s\n",strerror(errno));
+    printf("Press the enter key to continue.\n");
+    getchar();
   }
   else
   {
@@ -111,7 +112,9 @@ static FILE* init_log(const char*filename,int argc, char**argv)
 	  fprintf(f_file," %s", argv[i]);
 	fprintf(f_file,"\n");
 	fflush(f_file);
+#ifdef HAVE_DUP2
 	dup2(fileno(f_file),2);
+#endif
   }
   return f_file;
 }
@@ -133,7 +136,11 @@ void aff_copy(WINDOW *window)
   wclear(window);
   keypad(window, TRUE); /* Need it to get arrow key */
   wmove(window,0,0);
-  wdoprintf(window, "TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
+  wdoprintf(window, "TestDisk %s, Data Recovery Utility, %s",VERSION,TESTDISKDATE);
+  wmove(window,1,0);
+  wdoprintf(window,"Christophe GRENIER <grenier@cgsecurity.org>");
+  wmove(window,2,0);
+  wdoprintf(window,"http://www.cgsecurity.org");
 }
 
 int main( int argc, char **argv )
@@ -147,6 +154,7 @@ int main( int argc, char **argv )
   int create_backup=0;
   t_list_disk *list_disk=NULL;
   t_list_disk *element_disk;
+  const char *cmd_device=NULL;
 #ifdef SOLARIS
   const t_arch_fnct *arch=&arch_sun;
 #else
@@ -168,7 +176,7 @@ int main( int argc, char **argv )
     return -1;
   }
 #endif
-#ifdef __CYGWIN__
+#if defined(__CYGWIN__) || defined(__MINGW32__)
   /* Average Windows user doesn't know how to specify a parameter */
   if(argc==1)
   {
@@ -204,6 +212,28 @@ int main( int argc, char **argv )
     else if((strcmp(argv[i],"/help")==0) || (strcmp(argv[i],"-help")==0) || (strcmp(argv[i],"--help")==0) ||
       (strcmp(argv[i],"/h")==0) || (strcmp(argv[i],"-h")==0))
       help=1;
+      /*
+    else if(strcmp(argv[i],"/cmd")==0)
+    {
+      if(i+2>=argc)
+	help=1
+      else
+      {
+	cmd_device=argv[i+1];
+	 TODO: handle
+	  analyse
+	  geometry,C,H,S
+	  write
+	  backup
+	  restore
+	  search
+	  rebuildbs
+	  rebuildbs,write
+	  cleanfat
+	  searchsb
+      }
+    }
+       */
     else
     {
       list_disk=insert_new_disk(list_disk,file_test_availability(argv[i],debug,arch));
@@ -212,43 +242,9 @@ int main( int argc, char **argv )
 	help=1;
       }
     }
-    if(create_log==1 && f_rapport==NULL)
-    {
-      /*    const char *ext2fs_version=NULL; */
-      f_rapport=init_log("testdisk.log",argc,argv);
-      ecrit_rapport("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
-#ifdef DJGPP
-      ecrit_rapport("Dos version");
-#elif defined(BSD)
-      ecrit_rapport("BSD version");
-#elif defined(LINUX)
-      ecrit_rapport("Linux version");
-#elif defined(__CYGWIN__)
-      ecrit_rapport("Windows version");
-#else
-      ecrit_rapport("Undefined OS");
-#endif
-#ifdef COMPILE_BY
-#ifdef COMPILE_HOST
-#ifdef COMPILE_TIME
-      ecrit_rapport(" (%s@%s, %s)",COMPILE_BY,COMPILE_HOST,COMPILE_TIME);
-#endif
-#endif
-#endif
-      ecrit_rapport("\n");
-      /*    ext2fs_get_library_version(ext2fs_version,NULL); */
-      /*    ecrit_rapport("%s\n\n",ext2fs_version); */
-#ifdef DEBUG
-      ecrit_rapport("Key down:      0x%x\n", KEY_DOWN);
-      ecrit_rapport("Key up:        0x%x\n", KEY_UP);
-      ecrit_rapport("Key left:      0x%x\n", KEY_LEFT);
-      ecrit_rapport("Key right:     0x%x\n", KEY_RIGHT);
-      ecrit_rapport("Key page down: 0x%x\n", KEY_NPAGE);
-      ecrit_rapport("Key page up:   0x%x\n", KEY_PPAGE);
-      ecrit_rapport("Key enter:     0x%x\n", KEY_ENTER);
-#endif
-    }
   }
+  if(cmd_device!=NULL && list_disk!=NULL)
+    help=1;
   printf("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
   if(help!=0)
   {
@@ -275,7 +271,43 @@ int main( int argc, char **argv )
 	    "\nIf you have problems with TestDisk or bug reports, please contacte me.\n");
     return 0;
   }
-  printf("Please wait...\n");
+  if(create_log==1 && f_rapport==NULL)
+  {
+    /*    const char *ext2fs_version=NULL; */
+    f_rapport=init_log("testdisk.log",argc,argv);
+    ecrit_rapport("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
+#ifdef DJGPP
+    ecrit_rapport("Dos version");
+#elif defined(BSD)
+    ecrit_rapport("BSD version");
+#elif defined(LINUX)
+    ecrit_rapport("Linux version");
+#elif defined(__CYGWIN__) || defined(__MINGW32__)
+    ecrit_rapport("Windows version");
+#else
+    ecrit_rapport("Undefined OS");
+#endif
+#ifdef COMPILE_BY
+#ifdef COMPILE_HOST
+#ifdef COMPILE_TIME
+    ecrit_rapport(" (%s@%s, %s)",COMPILE_BY,COMPILE_HOST,COMPILE_TIME);
+#endif
+#endif
+#endif
+    ecrit_rapport("\n");
+    /*    ext2fs_get_library_version(ext2fs_version,NULL); */
+    /*    ecrit_rapport("%s\n\n",ext2fs_version); */
+#ifdef DEBUG
+    ecrit_rapport("Key down:      0x%x\n", KEY_DOWN);
+    ecrit_rapport("Key up:        0x%x\n", KEY_UP);
+    ecrit_rapport("Key left:      0x%x\n", KEY_LEFT);
+    ecrit_rapport("Key right:     0x%x\n", KEY_RIGHT);
+    ecrit_rapport("Key page down: 0x%x\n", KEY_NPAGE);
+    ecrit_rapport("Key page up:   0x%x\n", KEY_PPAGE);
+    ecrit_rapport("Key enter:     0x%x\n", KEY_ENTER);
+#endif
+  }
+printf("Please wait...\n");
   {
     const char *locale;
     locale = setlocale (LC_ALL, "");
@@ -291,18 +323,20 @@ int main( int argc, char **argv )
   if(list_disk==NULL)
   {
     printf("No harddisk found\n");
-#ifdef __CYGWIN__
+#if defined(__CYGWIN__) || defined(__MINGW32__)
     printf("You need to be administrator to use TestDisk\n");
     printf("Under Win9x, use the DOS version instead\n");
     ecrit_rapport("You need to be administrator to use TestDisk\n");
 #else
 #ifndef DJGPP
+#ifdef HAVE_GETEUID
     if(geteuid()!=0)
     {
       printf("You need to be root to use TestDisk\n");
       ecrit_rapport("You need to be root to use TestDisk\n");
     }
 #endif
+#endif
 #endif
   }
 #ifdef DJGPP
@@ -321,7 +355,6 @@ int main( int argc, char **argv )
   }
   printf("\n");
   ecrit_rapport("\n");
-
   if(do_list!=0)
   {
     for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
@@ -343,10 +376,7 @@ int main( int argc, char **argv )
   }
   else
   {
-    if(do_curses_testdisk(debug,dump_ind,list_disk,saveheader,&arch))
-    {
-      printf("TestDisk need 25 lines to work.\nPlease enlarge the terminal and restart TestDisk.\n");
-    }
+    do_curses_testdisk(debug,dump_ind,list_disk,saveheader,&arch);
   }
   for(element_disk=list_disk;element_disk!=NULL;)
   {
