@@ -24,7 +24,6 @@
 #include <config.h>
 #endif
  
-#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -92,7 +91,7 @@ int recover_xfs(t_param_disk *disk_car, const struct xfs_sb *sb,t_partition *par
   }
 
   partition->part_size = (uint64_t)be64(sb->sb_dblocks) * be32(sb->sb_blocksize);
-  partition->part_type = (unsigned char)0x83;
+  partition->part_type = (unsigned char)P_LINUX;
   set_xfs_info(disk_car,sb,partition,debug,dump_ind);
   return 0;
 }
